<?php

/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */

namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Block;

use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Catalog\Model\CategoryFactory;
use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\Product\Visibility;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Framework\App\Request\Http;
use Magento\Framework\View\Element\Template;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Theme\Block\Html\Header\Logo;

class ActionableGoogleAnalytics extends Template
{
    /**
     * @var Data
     */
    public $_helper;
    /**
     * @var Http
     */
    protected $request;
    /**
     * @var
     */
    protected $blockFactory;
    /**
     * @var Logo
     */
    protected $_logo;
    /**
     * @var
     */
    protected $productCollectionFactory;
    /**
     * @var Visibility
     */
    protected $productVisibility;
    /**
     * @var Status
     */
    protected $productStatus;
    /**
     * @var CategoryFactory
     */
    private $_categoryFactory;
    /**
     * @var CollectionFactory
     */
    private $_productCollectionFactory;
    /**
     * @var \Magento\Directory\Model\Currency
     */
    private $_currency;
    /**
     * GoogleAnalyticsAndShoppingIntegration constructor.
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param CollectionFactory $productCollectionFactory
     * @param CategoryFactory $categoryFactory
     * @param Http $request
     * @param Data $helper
     * @param Logo $logo
     * @param Status $productStatus
     * @param Visibility $productVisibility
     * @param StoreManagerInterface $storeManager
     * @param \Magento\Directory\Model\Currency $currency
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        CollectionFactory $productCollectionFactory,
        CategoryFactory $categoryFactory,
        Http $request,
        Data $helper,
        Logo $logo,
        Status $productStatus,
        Visibility $productVisibility,
        StoreManagerInterface $storeManager,
        \Magento\Directory\Model\Currency $currency,
        array $data = []
    ) {
        $this->_productCollectionFactory = $productCollectionFactory;
        $this->_categoryFactory = $categoryFactory;
        $this->_logo = $logo;
        $this->_helper = $helper;
        $this->request = $request;
        $this->productStatus = $productStatus;
        $this->productVisibility = $productVisibility;
        $this->_storeManager = $storeManager;
        $this->_currency = $currency;
        parent::__construct($context, $data);
    }
    /**
     * is EE enable
     *
     * @return boolean
     */
    public function isEnhanceEcommerceEnable()
    {
        return $this->_helper->isEnhanceEcommerceEnable();
    }
    /**
     * is gtag snippet added or not
     *
     * @return boolean
     */
    public function isGtagSnippetAdd()
    {
        return $this->_helper->isGtagSnippetAdd();
    }

    /**
     * Check extension is enable
     *
     * @return boolean
     */
    public function isEnabled()
    {
        return $this->_helper->isEnabled();
    }
    /**
     * Tracking mode Universal analytics,GA4,Both
     *
     * @return void
     */
    public function mode()
    {
        return $this->_helper->mode();
    }
    /**
     * tracking method for Universal Analytics gtag.js,analytics.js
     *
     * @return void
     */
    public function method()
    {
        return $this->_helper->method();
    }
    /**
     * Get measurment ID
     *
     * @return void
     */
    public function getMeasurmentIDGA4()
    {
        return $this->_helper->getMeasurmentIDGA4();
    }
    /**
     * Get both UA ID
     *
     * @return void
     */
    public function getUaIDBoth()
    {
        return $this->_helper->getUaIDBoth();
    }

    /**
     * Get both GA4 ID
     *
     * @return void
     */
    public function getMeasurmentIdBoth()
    {
        return $this->_helper->getMeasurmentIdBoth();
    }
    /**
     * Get UA ID
     *
     * @return void
     */
    public function getUaId()
    {
        return $this->_helper->getUaID();
    }

    public function checkDFEnabled()
    {
        return $this->_helper->checkDFEnabled();
    }

    public function getGtm()
    {
        return $this->_helper->getGtm();
    }

    public function checkIPAnonymization()
    {
        return $this->_helper->checkIP_enabled();
    }

    public function getLinkAttribution()
    {
        return $this->_helper->getLinkAttribution();
    }

    public function getDisableAdsFeature()
    {
        return $this->_helper->getDisableAdsFeature();
    }

    public function checkClientIdEnabled()
    {
        return $this->_helper->checkClientIdEnabled();
    }

    public function getConversionSnippet()
    {
        return $this->_helper->getConversionSnippet();
    }

    public function checkOptOutEnabled()
    {
        return $this->_helper->checkOptOutEnabled();
    }

    public function getBestSeller()
    {
        return $this->_helper->getBestSeller();
    }

    public function FBPixelEnabled()
    {
        return $this->_helper->checkFbpixelEnabled();
    }

    public function adwordsEnabled()
    {
        return $this->_helper->checkAdwords_Enabled();
    }

    public function getAdwordsID()
    {
        return $this->_helper->getAdwordsID();
    }

    public function getSenToCode()
    {
        return $this->_helper->getSenToCode();
    }

    public function getOptimizeIP()
    {
        return $this->_helper->getOptimizeIP();
    }

    public function getSubscriptionId()
    {
        return $this->_helper->getSubscriptionId();
    }

    public function getConversionLabel()
    {
        $conversionLable = str_replace('Order Conversion #', '', $this->_helper->getConversionLabel());
        return $conversionLable;
    }

    public function getConversionId()
    {
        return $this->_helper->getConversionId();
    }

    public function getFbPixelID()
    {
        return $this->_helper->getFBPixelID();
    }

    public function getMagentoVersion()
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $productMetadata = $objectManager->get('Magento\Framework\App\ProductMetadataInterface');
        $version = $productMetadata->getVersion();
        return $version;
    }

    public function getRemarketTag()
    {
        return $this->_helper->getRemarketTag();
    }

    public function getDynamicRemarketTag()
    {
        return $this->_helper->getDynamicRemarketTag();
    }

    public function getGoogleAdsConversios()
    {
        return $this->_helper->getGoogleAdsConversios();
    }

    /**
     * Get Full page action name
     *
     * @return void
     */
    public function getAction()
    {
        $get_action = $this->request->getFullActionName();

        if (method_exists($this, $get_action)) {
            $data = $this->$get_action();
            return [$data, $get_action];
        } else {
            return $get_action;
        }
    }

    /**
     * Get custom action for home page
     *
     * @return void
     */
    public function getCustomAction()
    {
        return $this->request->getFullActionName();
    }

    public function getLocalCurrency()
    {
        return $this->_helper->getCurrencyCode();
    }

    /**
     * Get User ID
     *
     * @return void
     */
    public function getUserID()
    {
        $user_id = '';
        if ($this->_helper->getUser_ID()) {
            $user_id = $this->_helper->getUser_ID();
        }
        return base64_encode($user_id);
    }

    public function getTotalOrder()
    {
        return $this->_helper->getTotalOrder();
    }

    public function getTotal()
    {
        return $this->_helper->getFbTotal();
    }

    /**
     * Check is admin logged in
     *
     * @return boolean
     */
    public function isAdminLogin()
    {
        return $this->_helper->isAdminLogin();
    }

    /**
     * Catalog search products data
     *
     * @return void
     */
    protected function catalogsearch_result_index()
    {
        $searchProducts = $this->_helper->getSearchProducts();
        return json_encode($searchProducts);
    }

    /**
     * Catalog category view data
     *
     * @return void
     */
    protected function catalog_category_view()
    {
        $t_getCategoryProduct = $this->_helper->getCategoryProduct();
        return json_encode($t_getCategoryProduct);
    }

    /**
     * Catalog product view data
     *
     * @return void
     */
    protected function catalog_product_view()
    {
        $tvc_product_details = $this->_helper->getProductDetails();
        $tvc_related_products = $this->_helper->getRalatedProducts();
        $tvc_upsell_products = $this->_helper->getUpSellProducts();
        // $tvc_crosssell_products = $this->_helper->getCrossSellProducts();
        array_push($tvc_product_details, $tvc_related_products, $tvc_upsell_products);
        return json_encode($tvc_product_details);
    }

    /**
     * Catalog cart index products data
     *
     * @return void
     */
    protected function checkout_cart_index()
    {
        $tvc_cart_items['cart'] = $this->_helper->getCartpageInfo();
        $tvc_crosssell_products['cross'] = $this->_helper->getCrossSellProducts();
        array_push($tvc_cart_items, $tvc_crosssell_products);
        return json_encode($tvc_cart_items);
    }

    /**
     * Get checkout products data
     *
     * @return void
     */
    protected function checkout_index_index()
    {
        $tvc_cart_items = $this->_helper->getCartpageInfo();
        return json_encode($tvc_cart_items);
    }

    /**
     * Get order data on thank you page
     *
     * @return void
     */
    protected function checkout_onepage_success()
    {
        $tvc_order_obj = $this->_helper->getOrderDetails();
        //print_r($tvc_order_obj);echo "====<br/>======";
        $tvc_trans_detail = $this->_helper->getTransactionDetails();
        //print_r($tvc_trans_detail);        
        array_push($tvc_order_obj, $tvc_trans_detail);
        return json_encode($tvc_order_obj);
    }

    /**
     * Catgory products on home page
     *
     * @return void
     */
    public function getHomePageCatProd()
    {
        $tvc_hp_products = $this->_helper->getHpCatProducts();
        return json_encode($tvc_hp_products);
    }

    /**
     * Products data by brand/manufacturer
     *
     * @return void
     */
    public function getProductsByManufacturer()
    {
        $tvc_hp_brand_prod = $this->_helper->getProductsByManufacturer();
        return json_encode($tvc_hp_brand_prod);
    }

    /**
     * Get best seller products
     *
     * @return void
     */
    public function getBestSellerProducts()
    {
        $tvc_best_seller = $this->_helper->getBestSellerProducts();
        return json_encode($tvc_best_seller);
    }

    /**
     * Check form field tracking enable
     *
     * @return void
     */
    public function formFieldTrackingEnable()
    {
        return $this->_helper->formFieldTrackingEnable();
    }

    /**
     * Is customer logged in or not
     *
     * @return boolean
     */
    public function isCustomerLoggedIn()
    {
        return $this->_helper->check_IsLoggedIn();
    }

    public function getCartCrossSellProducts()
    {
        return $this->_helper->getCartCrossSellProducts();
    }

    /**
     * Check if current url is url for home page
     */
    public function isHomePage()
    {
        return $this->_logo->isHomePage();
    }
    /**
     * Get current store currency code
     *
     * @return string
     */
    public function getCurrentCurrencyCode()
    {
        return $this->_storeManager->getStore()->getCurrentCurrencyCode();
    }
}
