<?php

/**
 * Conversios Software
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */

namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\Index;

use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\CustomApi;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\Product\Grid;
use Magento\Framework\Notification\MessageInterface;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Cron\AutoSyncModel;
use Magento\Framework\HTTP\Client\Curl;

class Index extends \Magento\Framework\View\Element\Template
{

    const XML_SUBSCRIPTION_ID = 'tatvic_ee/user/subscription_id';

    const XML_PATH_NEXT_PAGE_TOKEN = 'tatvic/nextPageToken';
    /**
    * @var Curl
    */
    protected $curl;
    /**
     * @var Data
     * Get helper data 
     */
    public $_helper;
    /**
     * Undocumented variable
     * Get custom api data from helper
     * @var [custom helper]
     */
    protected $_categoryHelper;
    protected $categoryFlatConfig;
    /**
     * @var array
     */
    protected $categoryTree;
    /**
     * @var string
     */
    protected $_template = 'actionablegoogleanalytics_index_index.phtml';

    protected $_attributeFactory;

    public $adminCategoryTree;

    protected $objectManager;

    protected $apiDomain;

    protected $customHelper;

    protected $_scopeConfig;

    protected $limit;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManagerInterface;

    protected $storeScope;

    protected $_resources;

    protected $messageManager;

    protected $objDate;

    /**
     * @var \Magento\AdminNotification\Model\Inbox
     */
    protected $adminNotificationInbox;

    protected $cronTest;

    /**
     * Index class construct
     *
     * @param \Magento\Backend\Block\Widget\Context $context
     * @param \Magento\Catalog\Block\Adminhtml\Category\Tree $adminCategoryTree
     * @param \Magento\Catalog\Model\ResourceModel\Eav\Attribute $attributeFactory
     * @param \Magento\Catalog\Helper\Category $categoryHelper
     * @param \Magento\Catalog\Model\Indexer\Category\Flat\State $categoryFlatState
     * @param Data $helper
     * @param CustomApi $customHelper
     * @param ScopeConfigInterface $_scopeConfig
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param \Magento\Store\Model\StoreManagerInterface $StoreManagerInterface
     * @param Grid $limit
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Widget\Context $context,
        \Magento\Catalog\Block\Adminhtml\Category\Tree $adminCategoryTree,
        \Magento\Catalog\Model\ResourceModel\Eav\Attribute $attributeFactory,
        \Magento\Catalog\Helper\Category $categoryHelper,
        \Magento\Catalog\Model\Indexer\Category\Flat\State $categoryFlatState,
        Data $helper,
        CustomApi $customHelper,
        ScopeConfigInterface $_scopeConfig,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \Magento\Store\Model\StoreManagerInterface $StoreManagerInterface,
        Grid $limit,
        AutoSyncModel $cronTest,
        \Magento\AdminNotification\Model\Inbox $adminNotificationInbox,
        
        Curl $curl,
        array $data = []
    ) {
        $this->objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $this->adminCategoryTree = $adminCategoryTree;
        $this->_attributeFactory = $attributeFactory;
        $this->_categoryHelper = $categoryHelper;
        $this->categoryFlatConfig = $categoryFlatState;
        $this->_helper = $helper;
        $this->customHelper = $customHelper;
        $this->_scopeConfig = $_scopeConfig;
        $this->messageManager = $messageManager;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->storeManagerInterface = $StoreManagerInterface;
        $this->limit = $limit;
        $this->_helper->cacheClean();
        $this->adminNotificationInbox = $adminNotificationInbox;
        $this->_resources = $this->objectManager->get('Magento\Framework\App\ResourceConnection');
        $this->objDate = $this->objectManager->create('Magento\Framework\Stdlib\DateTime\DateTime');
        $this->cronTest = $cronTest;
        $this->getLoadNotification();
        $this->curl = $curl;
        parent::__construct($context, $data);
    }

    public function cronTest()
    {
        return $this->cronTest->productsAutoSync();
    }

    /**
     * Get last product sync list update time
     *
     * @return void
     */
    public function getLastUpdateTime()
    {
        return $this->_helper->getLastUpdateTime();
    }

    /**
     * Get Subscribe user Email ID
     */
    public function getSubscribeEmail()
    {
        return $this->_helper->subscribeEmail();
    }

    /**
     * Get product refresh nextPage token limit value
     * @return void
     */
    public function getProductsRefresh()
    {
        return $this->limit->getLimit();
    }

    /**
     * Refresh products list controller url
     * @return void
     */
    public function refreshProducts()
    {
        return $this->getUrl('actionablegoogleanalytics/RefreshProducts');
    }

    /**
     * Category mapping controller url
     * @return void
     */
    public function mappAjaxUrl()
    {
        return $this->getUrl('actionablegoogleanalytics/CategoryMapping');
    }

    /**
     * SubcategoryMap controller
     *
     * @return void
     */
    public function subCatAjaxUrl()
    {
        return $this->getUrl('actionablegoogleanalytics/SubCategoryMap');
    }

    /**
     * Data push to MCA controller
     *
     * @return void
     */
    public function ajaxDataPush()
    {
        return $this->getUrl('actionablegoogleanalytics/ajaxMapData');
    }

    public function ajaxDataTruncateUrl()
    {
        return $this->getUrl('actionablegoogleanalytics/TruncateData');
    }

    /**
     * Product sync controller
     *
     * @return void
     */
    public function productsSync()
    {
        return $this->getUrl('actionablegoogleanalytics/productsSync');
    }

    /**
     * Get merchant center attribute list from helper/json/gmc_attrbutes.json file
     *
     * @return void
     */
    public function getMerchantAttributes()
    {
        return $this->customHelper->getMerchantAttributes();
    }

    /**
     * Get country list from helper/json/countries.json file
     *
     * @return void
     */
    public function getCountries()
    {
        return $this->customHelper->getCountries();
    }

    /**
     * Get mapped category from core_config_data DB table
     *
     * @return void
     */
    public function getMappedCategories()
    {
        return $this->_helper->getMappedCategories();
    }

    /**
     * Get mapped attributes
     *
     * @return void
     */
    public function getMappedAttributes()
    {
        return $this->_helper->getMappedAttributes();
    }

    /**
     * Get max limit of subscription plan
     * Retrieve system message severity
     * Possible default system message types:
     * - MessageInterface::SEVERITY_CRITICAL
     * - MessageInterface::SEVERITY_MAJOR
     * - MessageInterface::SEVERITY_MINOR
     * - MessageInterface::SEVERITY_NOTICE
     * @return void
     */
    public function getLoadNotification()
    {
        $this->adminNotificationInbox->parse(
            [
                [
                    'severity' => MessageInterface::SEVERITY_NOTICE,
                    'date_added' => $this->objDate->gmtDate(),
                    'title' => "Congratulations..!! You are one step closer",
                    'description' => "Thanks for installing the new avatar of Actionable Google Analytics for Magento 2 Extension. \nExplore the full potential of Google Analytics, Google Ads and Google shopping by setting up all your Google accounts\n and take data driven decisions to scale your eCommerce business faster."
                ],
            ]
        );
        $this->adminNotificationInbox->parse(
            [
                [
                    'severity' => MessageInterface::SEVERITY_NOTICE,
                    'date_added' => $this->objDate->gmtDate(),
                    'title' => "Share your feedback",
                    'description' => "Your feedback is very important to us.\n Please write about your experience and the new feature requests on https://marketplace.magento.com/conversios-googleanalyticsandshoppingintegration.html#bazaarvoice.reviews.tab"
                ],
            ]
        );
        return $this->_helper->getMaxLimit();
    }

    /**
     * Get statistics value for display on mapping template
     *
     * @return void
     */
    public function getStatistics()
    {
        $statistics = [
            'total' => $this->getStatisticsTotal(),
            'disaproved' => $this->getStatisticsStatusDisapprove(),
            'approved' => $this->getStatisticsStatusAapprove(),
            'pending' => $this->getStatisticsStatusPending()
        ];

        return $statistics;
    }

    /**
     * Get category data for mapping tree
     *
     * @param [type] $parentCatId
     * @return void
     */
    public function getCategoryData($parentCatId)
    {
        $categoryRepository = $this->objectManager->create('Magento\Catalog\Model\CategoryRepository');
        $parentcategories = $categoryRepository->get($parentCatId);
        $categories = $parentcategories->getChildrenCategories();
        $i = 0;
        $ChildCategoryValue = [];

        foreach ($categories as $category) {
            $count = $category->getProductCollection()->count();
            $ChildCategoryValue[$i] = ['name' => $category->getName(), 'id' => $category->getId(), 'count' => $count];
            $childCat = $this->getCategoryData($category->getId());
            if ($childCat) {
                $ChildCategoryValue[$i]['child'] = $childCat;
            }
            $i++;
        }
        return $ChildCategoryValue;
    }

    /**
     * Get store main category
     *
     * @param [type] $cat
     * @return void
     */
    public function getMainCategory($cat)
    {
        return $this->objectManager->create('Magento\Catalog\Model\Category')->load($cat);
    }

    /**
     * Get statistical total
     *
     * @return void
     */
    public function getStatisticsTotal()
    {
        $connection = $this->_resources->getConnection();
        $table = $this->_resources->getTableName('product_sync_list');
        $getData = 'SELECT * FROM '. $table;
        $allData = $connection->fetchAll($getData);
        return count($allData);
    }

    public function getUnSyncProducts()
    {
        $connection = $this->_resources->getConnection();
        $syncTable = $this->_resources->getTableName('pre_sync_data');
        $getData = 'SELECT * FROM  '. $syncTable .' WHERE status = 1 ORDER BY id ASC';
        return count($connection->fetchAll($getData));
    }

    /**
     * Get statistics status disapprove from table
     *
     * @return void
     */
    public function getStatisticsStatusDisapprove()
    {
        $connection = $this->_resources->getConnection();
        $table = $this->_resources->getTableName('product_sync_list');
        $getData = 'SELECT * FROM ' . $table . ' WHERE `googleStatus` = "disapproved"';
        $allData = $connection->fetchAll($getData);
        return count($allData);
    }

    /**
     * Get statistics status approve from table
     *
     * @return void
     */
    public function getStatisticsStatusAapprove()
    {
        $connection = $this->_resources->getConnection();
        $table = $this->_resources->getTableName('product_sync_list');
        $getData = 'SELECT * FROM ' . $table . ' WHERE `googleStatus` = "approved"';
        $allData = $connection->fetchAll($getData);
        return count($allData);
    }

    /**
     * Get statistics status pending from table
     *
     * @return void
     */
    public function getStatisticsStatusPending()
    {
        $connection = $this->_resources->getConnection();
        $table = $this->_resources->getTableName('product_sync_list');
        $getData = "SELECT * FROM " . $table . " WHERE `googleStatus` = 'pending'";
        $allData = $connection->fetchAll($getData);
        return count($allData);
    }

    /**
     * Get products attributes
     *
     * @return void
     */
    public function getProductAttributes()
    {
        $attributeInfo = $this->_attributeFactory->getCollection();
        $attributess = [];
        foreach ($attributeInfo as $attributes) {
            $attribute = [];
            $attribute['attribute_id'] = $attributes->getAttributeId();
            $attribute['attribute_name'] = $attributes->getName();
            // You can get all fields of attribute here
            array_push($attributess, $attribute);
        }
        return $attributess;
    }

    public function getDomain()
    {
        return $this->_helper->getApiUrl();
    }

    public function getBaseUrl()
    {
        return $this->storeManagerInterface->getStore()->getBaseUrl();
    }

    function getStoreLanguage()
    {
        return $this->_helper->getLocaleCode();
    }

    public function getCountryCode()
    {
        return $this->_helper->getCountry();
    }

    public function getCustomerId()
    {
        return $this->_helper->getCurrentCustomerId();
    }

    public function getStoreProductCount()
    {
        return $this->_helper->getStoreProductCount();
    }

    public function getMerchantId()
    {
        return $this->_helper->getMerchantId();
    }

    public function getSubscriptionId()
    {
        return $this->_helper->getSubscriptionId();
    }
    public function getGMCCustomerId()
    {
        return $this->_helper->getCustomerId();
    }

    public function getAccountId()
    {
        return $this->_tvc_ga_options['advance']['merchant_id'];
    }

    /**
     * Returns the campaign categories from a selected country
     */
    public function tvcGetGmcCategories()
    {
        $country_code = $this->_helper->getCountry();
        $customer_id = $this->_helper->getCurrentCustomerId();
        $parent = 'parent';

        $header = [
            "Authorization: Bearer MTIzNA==",
            "content-type: application/json"
        ];

        $data = [
            'customer_id' => $customer_id,
            'country_code' => $country_code,
            'parent' => '',
        ];

        $postData = json_encode($data);

        $curl_url = $this->getDomain() . '/products/gmc-categories';

        /*$ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_URL => $curl_url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_HTTPHEADER => $header,
            CURLOPT_POSTFIELDS => $postData,
        ));
        $response = curl_exec($ch);
        $response = json_decode($response);
        return $response;*/

        //set curl options
        $this->curl->setOption(CURLOPT_HEADER, 0);
        $this->curl->setOption(CURLOPT_TIMEOUT, 60);
        $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);
        //set curl header
        $this->curl->addHeader($header);
        //post request with url and data
        $this->curl->post($curl_url, $postData);
        //read response
        $response = $this->curl->getBody();
        return $response;
    }

    public function getRootCatId()
    {
        return $this->storeManagerInterface->getStore()->getRootCategoryId();
    }

    public function getContentLanguage()
    {
        return $this->customHelper->getContentLanguage();
    }

    /**
     * Category mapping selectore dropdowns
     *
     * @param [type] $identifier
     * @param [type] $id
     * @param [type] $start_visible
     * @param [type] $ee_prod_mapped_cats
     * @return void
     */
    public function categoryMappingSelector($identifier, $id, $start_visible, $ee_prod_mapped_cats)
    {
        $ee_prod_mapped_cats = $this->getMappedCategories();

        $display = 'initial';
        $ident = '-1' !== $id ? $identifier . '-' . $id : $identifier;
        $category_levels = 6;
        if (isset($ee_prod_mapped_cats[$id]['id']) && isset($ee_prod_mapped_cats[$id]['name']) && $ee_prod_mapped_cats[$id]['id'] && $ee_prod_mapped_cats[$id]['name']) {
            $cat_name = $ee_prod_mapped_cats[$id]['name'];
            $cat_id = $ee_prod_mapped_cats[$id]['id'];
            $html_code = '<div class="column" id="category-selector-' . $ident . '" style="display:' . $display . '">
                    <div id="selected-categories">
                    <input type="hidden" name="category-' . $id . '" id="storeCategory" value="' . $id . '">
                    <input type="hidden" name="category-' . $id . '" id="category-' . $id . '" value="' . $cat_id . '">
                    <input type="hidden" name="category-name-' . $id . '" id="category-name-' . $id . '" value="' . $cat_name . '">
                    </div>
                    <label id="label-' . $ident . '_0">' . $cat_name . '</label><span style="margin-left: 15px; color: blue;" class="change_prodct_feed_cat" data-cat-id="' . $id . '" data-id="' . $ident . '_0">Edit</span>
                    <select style="display:none;" class="form-control cat-mapping-select select admin__control-select" id="' . $ident . '_0" data-id="' . $id . '"></select>';
            for ($i = 1; $i < $category_levels; $i++) {
                $html_code .= '<select class="cat-mapping-select select admin__control-select" id="' . $ident . '_' . $i . '" data-value="0" data-id="' . $id . '" style="display:none;"></select>';
            }
        } else {
            $html_code = '<div class="column" id="category-selector-' . $ident . '" style="display:' . $display . '">
                    <div id="selected-categories">
                    <input type="hidden" name="category-' . $id . '" id="category-' . $id . '" value="">
                    <input type="hidden" name="category-name-' . $id . '" id="category-name-' . $id . '" value="">
                    </div>
                    <select class="form-control cat-mapping-select  select admin__control-select" id="' . $ident . '_0" data-id="' . $id . '"></select>';

            for ($i = 1; $i < $category_levels; $i++) {
                $html_code .= '<select class="cat-mapping-select  select admin__control-select" id="' . $ident . '_' . $i . '" data-value="0" data-id="' . $id . '" style="display:none;"></select>';
            }
        }
        $html_code .= '</div>';
        return $html_code;
    }

    /**
     * Get subscription data
     *
     * @param [type] $subId
     * @return void
     */
    public function getAnalyticsData($subId)
    {
        $subscriptionId = $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
        $subsId = !empty($subId) ? $subId : $subscriptionId;
        $header = array(
            "Authorization: Bearer MTIzNA==",
            "content-type: application/json",
        );
        $curl_url = $this->_helper->getApiUrl() . "/customer-subscriptions/subscription-detail";
        $postData = [
            'subscription_id' => $subsId,
            'domain' => $this->getDomain(),
        ];
        $postData = json_encode($postData);
        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_URL => $curl_url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_TIMEOUT => 1000,
            CURLOPT_HTTPHEADER => $header,
            CURLOPT_POSTFIELDS => $postData,
        ));
        $response = curl_exec($ch);
        $response = json_decode($response);

        return $response;
    }

    public function getConfigUrl()
    {
        return $this->getUrl('adminhtml/system_config/edit/section/settings/');
    }

    public function getMagentoSvg()
    {
        return $this->getViewFileUrl('Conversios_GoogleAnalyticsAndShoppingIntegration::images/magento-icon.svg');
    }

    public function getGmcSvg()
    {
        return $this->getViewFileUrl('Conversios_GoogleAnalyticsAndShoppingIntegration::images/google-shopping.svg');
    }
    public function getInfoIcon()
    {
        return $this->getViewFileUrl('Conversios_GoogleAnalyticsAndShoppingIntegration::images/information.svg');
    }
    public function getRefreshIcon()
    {
        return $this->getViewFileUrl('Conversios_GoogleAnalyticsAndShoppingIntegration::images/refresh_logo.png');
    }
    public function getSignOrganeTik()
    {
        return $this->getViewFileUrl('Conversios_GoogleAnalyticsAndShoppingIntegration::images/orange-tik.png');
    }
}
