<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\Product;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use PHPUnit\Framework\Exception;

class Grid extends \Magento\Backend\Block\Widget\Grid\Extended
{

    const XML_PATH_NEXT_PAGE_TOKEN = 'tatvic/nextPageToken';

    /**
     * @var \Magento\Framework\Module\Manager
     */
    protected $moduleManager;

    /**
     * @var \Conversios\GoogleAnalyticsAndShoppingIntegration\Model\ProductFactory
     */
    protected $_productFactory;

    /**
     * @var \Conversios\GoogleAnalyticsAndShoppingIntegration\Model\Status
     */
    protected $_status;

    protected $_resources;

    protected $_configWriter;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    protected $storeScope;

    protected $messageManager;

    protected $cacheManager;

    /**
     * Grid constructor
     *
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Backend\Helper\Data $backendHelper
     * @param \Conversios\GoogleAnalyticsAndShoppingIntegration\Model\ProductFactory $ProductFactory
     * @param \Conversios\GoogleAnalyticsAndShoppingIntegration\Model\Status $status
     * @param \Magento\Framework\Module\Manager $moduleManager
     * @param WriterInterface $scopeConfig
     * @param ScopeConfigInterface $_scopeConfig
     * @param \Magento\Framework\App\Cache\Manager $cacheManager
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Backend\Helper\Data $backendHelper,
        \Conversios\GoogleAnalyticsAndShoppingIntegration\Model\ProductFactory $ProductFactory,
        \Conversios\GoogleAnalyticsAndShoppingIntegration\Model\Status $status,
        \Magento\Framework\Module\Manager $moduleManager,
        WriterInterface $scopeConfig,
        ScopeConfigInterface $_scopeConfig,

        \Magento\Framework\App\Cache\Manager $cacheManager,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        array $data = []
    ) {
        $this->_productFactory = $ProductFactory;
        $this->_status = $status;

        $this->moduleManager = $moduleManager;

        $this->_scopeConfig = $_scopeConfig;
        $this->_configWriter = $scopeConfig;
        $this->messageManager = $messageManager;
        $this->cacheManager = $cacheManager;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->_resources = \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Framework\App\ResourceConnection');
        parent::__construct($context, $backendHelper, $data);
    }

    /**
     * @return void
     */
    protected function _construct()
    {
        parent::_construct();
        $this->setId('tatvicListingGrid');
        $this->setDefaultSort('id');
        $this->setDefaultDir('ASC');
        $this->setSaveParametersInSession(true);
        $this->setUseAjax(false);
        $this->setVarNameFilter('post_filter');
        $this->setDefaultLimit(20);
        $this->setFilterVisibility(false);
    }

    /**
     * @return $this
     */
    protected function _prepareCollection()
    {
        $collection = $this->_productFactory->create()->getCollection();
        $this->setCollection($collection);
        parent::_prepareCollection();
        return $this;
    }

    /**
     * @return \Magento\Backend\Block\Widget\Grid\Extended
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function _prepareColumns()
    {
        $this->addColumn(
            'imageLink',
            [
                'header' => __('Product Image'),
                'type' => 'image',
                'index' => 'image',
                'renderer' => '\Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\Product\Renderer\Image',
            ]
        );
        $this->addColumn(
            'name',
            [
                'header' => __('Product Name'),
                'index' => 'name',
            ]
        );
        $this->addColumn(
            'googleStatus',
            [
                'header' => __('Google Status'),
                'index' => 'googleStatus',
            ]
        );
        $this->addColumn(
            'issues',
            [
                'header' => __('Issues'),
                'index' => 'issues',
            ]
        );

        //$this->addExportType($this->getUrl('actionablegoogleanalytics/*/exportCsv', ['_current' => true]), __('CSV'));
        //$this->addExportType($this->getUrl('actionablegoogleanalytics/*/exportExcel', ['_current' => true]), __('Excel XML'));

        $block = $this->getLayout()->getBlock('grid.bottom.links');
        if ($block) {
            $this->setChild('grid.bottom.links', $block);
        }
        return parent::_prepareColumns();
    }

    /**
     * @return string
     */
    public function getGridUrl()
    {
        return $this->getUrl('actionablegoogleanalytics/*/index', ['_current' => true]);
    }
}
