<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\System\Config;

use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\CustomApi;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Backend\Block\Template\Context;
use Magento\Config\Block\System\Config\Form\Field;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Data\Form\Element\AbstractElement;

class Campaigns extends Field
{
    /**
     * Subscription ID path of core config data
     */

    const XML_SUBSCRIPTION_ID = 'tatvic_ee/user/subscription_id';

    /**
     * Define sync product button phtml file
     *
     * @var string
     */
    protected $_template = 'Conversios_GoogleAnalyticsAndShoppingIntegration::system/config/campaigns.phtml';

    public $_helper;

    protected $customHelper;

    protected $_scopeConfig;

    protected $directoryBlock;

    /**
     *
     * @param Context $context
     * @param Data $helper
     * @param ScopeConfigInterface $_scopeConfig
     * @param array $data
     */
    public function __construct(
        Context $context,
        Data $helper,
        CustomApi $customHelper,
        ScopeConfigInterface $_scopeConfig,
        \Magento\Directory\Block\Data $directoryBlock,
        array $data = []
    ) {
        $this->_helper = $helper;
        $this->_scopeConfig = $_scopeConfig;
        $this->customHelper = $customHelper;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->directoryBlock = $directoryBlock;
        $this->apiDomain = 'https://connect.tatvic.com/laravelapi/public/api';
        parent::__construct($context, $data);
    }

    /**
     * Get country list from helper/json/countries.json file
     *
     * @return void
     */
    public function getCountries()
    {
        return $this->customHelper->getCountries();
    }

    /**
     * Data push to MCA controller
     *
     * @return void
     */
    public function ajaxCreateCampaign()
    {
        return $this->getUrl('actionablegoogleanalytics/ajaxCreateCampaign');
    }

    /**
     * Get subscription ID
     *
     * @return void
     */
    public function getSubscritionId()
    {
        return $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
    }

    public function render(AbstractElement $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }
    /**
     *
     * @param AbstractElement $element
     * @return void
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * get product sync phtml file url
     *
     * @return void
     */
    public function getCustomUrl()
    {
        return $this->getUrl('actionablegoogleanalytics/index/index');
    }

    /**
     * Get button html
     *
     * @return void
     */
    public function getButtonHtml()
    {
        $button = $this->getLayout()->createBlock('Magento\Backend\Block\Widget\Button')->setData(['id' => 'new_product_sync_btn', 'class' => 'secondary ui-button ui-widget ui-state-default ui-corner-all', 'label' => __('Sync Products')]);
        return $button->toHtml();
    }

    /* New code*/

    /**
     * Get New Campaign button html
     *
     * @return void
     */
    public function getNewCampaignButtonHtml()
    {
        $button = $this->getLayout()->createBlock('Magento\Backend\Block\Widget\Button')->setData(['id' => 'new_campaign', 'class' => 'secondary ui-button ui-widget ui-state-default ui-corner-all', 'label' => __('New Campaign')]);
        return $button->toHtml();
    }

    //Edit-Campaign-Start
    public function getEditCampaignButtonHtml()
    {
        $button = $this->getLayout()->editBlock('Magento\Backend\Block\Widget\Button')->setData(['id' => 'edit_campaign', 'class' => 'secondary ui-button ui-widget ui-state-default ui-corner-all', 'label' => __('Edit Campaign')]);
        return $button->toHtml();
    }
    //Edit-Campaign-End

    public function getCurrency()
    {
        return $this->_helper->getCurrencyCode();
    }

    /**
     * Get country list from helper/json/countries.json file
     *
     * @return array
     */
    public function getCountry()
    {
        return $this->_helper->getCountry();
    }

    public function getConfigValues($path)
    {
        return $this->_scopeConfig->getValue($path, 'default', $this->storeScope);
    }

    public function getCampaignPerformanceDateRange()
    {
        return $this->getUrl('actionablegoogleanalytics/campaignPerformance/index');
    }
}
