<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\System\Config;

use Magento\Backend\Block\Template\Context;
use Magento\Config\Block\System\Config\Form\Field;
use Magento\Framework\Data\Form\Element\AbstractElement;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\CustomApi;

class CampaignsList  extends Field
{
    /**
     * Subscription ID path of core config data
     */

    const XML_SUBSCRIPTION_ID = 'tatvic_ee/user/subscription_id';

    /**
     * Define sync product button phtml file
     *
     * @var string
     */
    protected $_template = 'Conversios_GoogleAnalyticsAndShoppingIntegration::system/config/campaignsList.phtml';

    public $_helper;

    protected $_scopeConfig;

    protected $customHelper;

    /**
     *
     * @param Context $context
     * @param Data $helper
     * @param ScopeConfigInterface $_scopeConfig
     * @param array $data
     */
    public function __construct(
        Context $context, 
        Data $helper,
        CustomApi $customHelper,
        ScopeConfigInterface $_scopeConfig,
        array $data = []
    )
    {
        $this->_helper = $helper;
        $this->_scopeConfig = $_scopeConfig;
        $this->customHelper = $customHelper;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        parent::__construct($context, $data);
    }

    /**
     * Data push to MCA controller
     *
     * @return void
     */
    public function ajaxGetCampaignList()
    {
        return $this->customHelper->listShippingCampaign();
    }

    /**
     * Get subscription ID
     *
     * @return void
     */
    public function getSubscritionId()
    {
        return $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
    }

    public function render(AbstractElement $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }
    /**
     *
     * @param AbstractElement $element
     * @return void
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * get product sync phtml file url
     *
     * @return void
     */
    public function getCustomUrl()
    {
        return $this->getUrl('actionablegoogleanalytics/index/index');
    }

    /**
     * Get button html
     *
     * @return void
     */
    public function getButtonHtml()
    {
        $button = $this->getLayout()->createBlock('Magento\Backend\Block\Widget\Button')->setData(['id' => 'new_product_sync_btn','class' => 'secondary ui-button ui-widget ui-state-default ui-corner-all', 'label' => __('Sync Products'),]);
        return $button->toHtml();
    }
    
    /**
     * Get New Campaign button html
     *
     * @return void
     */
    public function getNewCampaignButtonHtml()
    {
        $button = $this->getLayout()->createBlock('Magento\Backend\Block\Widget\Button')->setData(['id' => 'new_campaign','class' => 'secondary ui-button ui-widget ui-state-default ui-corner-all', 'label' => __('New Campaign'),]);
        return $button->toHtml();
    }  

    /**
     * Get country list from helper/json/countries.json file
     *
     * @return array
     */
    public function getCountries()
    {
        return $this->customHelper->getCountries();
    }

    /**
     * Get country list from helper/json/countries.json file
     *
     * @return array
     */
    public function getCountry()
    {
        return $this->_helper->getCountry();
    }

    /**
     * Get Path list configuration files
     *
     * @return array
     */
    public function getConfigValues($path)
    {
        return $this->_scopeConfig->getValue($path, 'default', $this->storeScope);
    }

    public function getCurrency(){
        return $this->_helper->getCurrencyCode();
    }

    public function getEditCampaign()
    {
        return $this->getUrl('actionablegoogleanalytics/updateCampaign/index');
    }

    public function getCampaignDetails($cmpID)
    {
        return $this->customHelper->getCampaignDetails($cmpID);
    }

    public function deleteCampaign()
    {
        return $this->getUrl('actionablegoogleanalytics/deleteCampaign/index');
    }

    public function statuschangeCampaign()
    {
        return $this->getUrl('actionablegoogleanalytics/statuschangeCampaign/index');
    }

    public function getDeleteCampaign()
    {
        return $this->getUrl('actionablegoogleanalytics/index/index');
    } 

    /**
     * Data push to MCA controller
     *
     * @return void
     */
    public function ajaxCreateCampaign()
    {
        return $this->getUrl('actionablegoogleanalytics/ajaxCreateCampaign');
    }   

}
