<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\System\Config;

use Magento\Config\Block\System\Config\Form\Field;
use Magento\Backend\Block\Template\Context;
use Magento\Framework\Data\Form\Element\AbstractElement;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Framework\App\Config\ScopeConfigInterface;

class DomainClaim extends Field
{

    const XML_DOMAIN_CLAIM = 'tatvic_ee/google_shopping/domain_claim';

    protected $_helper;

    protected $_scopeConfig;

    protected $domainClaim;

    /**
     *
     * @param Context $context
     * @param Data $helper
     * @param ScopeConfigInterface $_scopeConfig
     * @param array $data
     */
    public function __construct(
        Context $context,
        Data $helper,
        ScopeConfigInterface $_scopeConfig,
        array $data = []
    ) {
        $this->_helper = $helper;
        $this->_scopeConfig = $_scopeConfig;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->domainClaim = $this->_scopeConfig->getValue(self::XML_DOMAIN_CLAIM, 'default', $this->storeScope);
        parent::__construct($context, $data);
    }

    /**
     * @return $this|\Magento\Config\Block\System\Config\Form\Field
     */
    protected function _prepareLayout()
    {
        parent::_prepareLayout();
        if (!$this->getTemplate()) {
            $this->setTemplate('system/config/domain_claim.phtml');
        }
        return $this;
    }

    /**
     * Render html
     *
     * @param AbstractElement $element
     * @return void
     */
    public function render(AbstractElement $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }
    protected function _getElementHtml(AbstractElement $element)
    {
        return $this->_toHtml();
    }

    public function getCustomUrl()
    {
        return $this->getUrl('actionablegoogleanalytics/domainclaim/index');
    }

    /**
     * Check is domain claim
     *
     * @return boolean
     */
    public function isDomainClaim()
    {
        return $this->domainClaim;
    }

    public function getButtonHtml()
    {
        $button = $this->getFailedSVG();
        return $button;
    }

    public function getSuccessSVG()
    {
        return $this->getViewFileUrl('Conversios_GoogleAnalyticsAndShoppingIntegration::images/site-ver-success.png');
    }

    public function getFailedSVG()
    {
        return $this->getViewFileUrl('Conversios_GoogleAnalyticsAndShoppingIntegration::images/exclaimation.png');
    }

    public function getRefreshLoaderDomain(){
        return $this->getViewFileUrl('Conversios_GoogleAnalyticsAndShoppingIntegration::images/refresh.png');
    }
}