<?php

/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */

namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\System\Config;

use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\CustomApi;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Backend\Block\Template\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\Stdlib\CookieManagerInterface;

class SignIn extends \Magento\Config\Block\System\Config\Form\Field
{

    const XML_SUBSCRIPTION_ID = 'tatvic_ee/user/subscription_id';

    const XML_PUBLIC_ACCESS_KEY = 'tatvic_ee/purchase_code/purchase_code';

    public $_helper;

    public $objectManager;

    public $connectUrl;

    protected $_configWriter;

    protected $token;

    protected $_scopeConfig;

    protected $apiDomain;

    public $storeScope;

    protected $apiHelper;

    protected $cacheManager;

    protected $messageManager;

    /**
     * @var \Magento\Framework\Stdlib\CookieManagerInterface
     */
    protected $cookieManager;

    protected $_urlInterface;

    /**
     * Sign in constructor
     *
     * @param Context $context
     * @param Data $helper
     * @param CustomApi $apiHelper
     * @param WriterInterface $scopeConfig
     * @param \Magento\Framework\Serialize\Serializer\Json $json
     * @param ScopeConfigInterface $_scopeConfig
     * @param \Magento\Framework\App\Cache\Manager $cacheManager
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param CookieManagerInterface $cookieManager
     * @param array $data
     */
    public function __construct(
        Context $context,
        Data $helper,
        CustomApi $apiHelper,
        WriterInterface $scopeConfig,
        \Magento\Framework\Serialize\Serializer\Json $json,
        ScopeConfigInterface $_scopeConfig,
        \Magento\Framework\App\Cache\Manager $cacheManager,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \Magento\Framework\UrlInterface $urlInterface,
        CookieManagerInterface $cookieManager,
        array $data = []
    ) {
        $this->cacheManager = $cacheManager;
        $this->_configWriter = $scopeConfig;
        $this->token = 'MTIzNA==';
        $this->objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $this->_helper = $helper;
        $this->json = $json;
        $this->_scopeConfig = $_scopeConfig;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->apiHelper = $apiHelper;
        $this->messageManager = $messageManager;
        $this->_urlInterface = $urlInterface;
        $this->cookieManager = $cookieManager;
        parent::__construct($context, $data);
    }

    /**
     * @return $this|\Magento\Config\Block\System\Config\Form\Field
     */
    protected function _prepareLayout()
    {
        parent::_prepareLayout();
        if (!$this->getTemplate()) {
            $this->setTemplate('system/config/fieldset/sign_in.phtml');
        }
        return $this;
    }

    public function subscribeEmail()
    {
        return $this->_helper->subscribeEmail();
    }

    /**
     * @param \Magento\Framework\Data\Form\Element\AbstractElement $element
     * @return string
     */
    protected function _getElementHtml(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        $this->siteVarificationAndDomainClaim();
        return $this->_toHtml();
    }

    public function getConnectUrl()
    {
        if (!empty($this->connectUrl)) {
            return $this->connectUrl;
        } else {
            $subscriptionId = $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
            return "https://" . $this->_helper->getConnectUrl() . "/config/ga_rdr_gmc.php?return_url=" . $this->_helper->getConnectUrl() . "/config/ads-analytics-form.php?domain=" . $this->getDomain() . "&country=" . $this->getCountry() . "&user_currency=" . $this->getCurrency() . "&subscription_id=" . $subscriptionId . "&confirm_url=" . $this->getCurrentUrl() . "&timezone=" . $this->getTimeZone() . "&app_id=2";
        }
    }

    /**
     * @return mixed
     */
    public function getDomain()
    {
        $storeManager = $this->objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        return $storeManager->getStore()->getBaseUrl();
    }

    public function getPublicAccessKey()
    {
        return $this->_scopeConfig->getValue(self::XML_PUBLIC_ACCESS_KEY, 'default', $this->storeScope);
    }

    public function getCountry()
    {
        return $this->_helper->getCountry();
    }

    /**
     * Site verification on sign in
     *
     * @return void
     */
    public function siteVarificationAndDomainClaim()
    {
        $subId = $this->getRequest()->getParam('subscription_id');
        $emailId = $this->getRequest()->getParam('g_mail');

        if (!empty($subId)) {
            $this->_configWriter->save('tatvic_ee/user/subscription_id', $subId);
            $this->_configWriter->save('tatvic_ee/subscription_enable/enable', 1);
            $this->_configWriter->save('tatvic_ee/purchase_code/enable', 1);

            $response = $this->getAnalyticsData($subId);
            if (session_status() === PHP_SESSION_NONE) {
                session_start();
            }

            if (!($response->error) == 1) {
                $data = $response->data;

                $this->_configWriter->save('tatvic_ee/advance/access_token', $data->access_token);
                $this->_configWriter->save('tatvic_ee/advance/refresh_token', $data->refresh_token);
                // save API data in DB
                $postData = [
                    'merchant_id' => $data->merchant_id,
                    'website_url' => $data->site_url,
                    'subscription_id' => $data->id,
                    'account_id' => $data->google_merchant_center_id,
                ];

                $directory = $this->objectManager->get('\Magento\Framework\Filesystem\DirectoryList');

                if ($data->is_site_verified == '0') {
                    $postData['method'] = "file";
                    $siteVerificationToken = $this->apiHelper->siteVerificationToken($postData);

                    if (isset($siteVerificationToken->error) && !empty($siteVerificationToken->errors)) {
                        //methd using tag
                        $postData['method'] = "meta";
                        $siteVerificationTokenTag = $this->apiHelper->siteVerificationToken($postData);

                        if (isset($siteVerificationTokenTag->data->token) && $siteVerificationTokenTag->data->token) {
                            $this->apiHelper->set_ee_additional_data(array("add_site_varification_tag" => 1, "site_varification_tag_val" => base64_encode($siteVerificationTokenTag->data->token)));
                            sleep(1);
                            $siteVerificationTag = $this->apiHelper->siteVerification($postData);
                            if (isset($siteVerificationTag->error) && !empty($siteVerificationTag->errors)) {
                                $this->messageManager->addError(__($siteVerificationTag->errors[0]));
                            } else {
                                $data->is_site_verified = '1';
                            }
                        }
                    } else {
                        if ($siteVerificationToken) {
                            $myFile = $directory->getRoot() . "/" . $siteVerificationToken->data->token;
                            if (!file_exists($myFile)) {
                                $fh = fopen($myFile, 'w+');
                                chmod($myFile, 0777);
                                $stringData = "google-site-verification: " . $siteVerificationToken->data->token;
                                fwrite($fh, $stringData);
                                fclose($fh);
                            }
                            $postData['method'] = "file";
                            $siteVerification = $this->apiHelper->siteVerification($postData);
                            if (isset($siteVerification->error) && !empty($siteVerification->errors)) {
                                //methd using tag
                                $postData['method'] = "meta";
                                $siteVerificationTokenTag = $this->apiHelper->siteVerificationToken($postData);

                                if (isset($siteVerificationTokenTag->data->token) && $siteVerificationTokenTag->data->token) {
                                    $this->apiHelper->set_ee_additional_data(array("add_site_varification_tag" => 1, "site_varification_tag_val" => base64_encode($siteVerificationTokenTag->data->token)));
                                    sleep(1);
                                    $siteVerificationTag = $this->apiHelper->siteVerification($postData);
                                    if (isset($siteVerificationTag->error) && !empty($siteVerificationTag->errors)) {
                                        $this->messageManager->addError(__($siteVerificationTag->errors[0]));
                                    } else {
                                        $data->is_site_verified = '1';
                                    }
                                }
                            } elseif ($siteVerification->data->site->identifier !== $this->getDomain()) {
                                $data->is_site_verified = '0';
                            } else {
                                $data->is_site_verified = '1';
                            }
                        }
                    }
                }

                if (!empty($data->google_ads_id) && $data->google_ads_conversion_tracking == 1) {
                    $dynamicRemarketingData = $this->apiHelper->getRemarketingSnippets($data->google_ads_id);

                    if (!empty($dynamicRemarketingData->data->id)) {
                        $this->_configWriter->save('tatvic_ee/google_ads/advenable', 1);
                        $this->_configWriter->save('tatvic_ee/google_ads/adv_id', $dynamicRemarketingData->data->id);
                        $this->_configWriter->save('tatvic_ee/google_ads/adv_snippet', $dynamicRemarketingData->data->snippets);
                        $this->_configWriter->save('tatvic_ee/google_ads/google_ads_conversion', $data->google_ads_conversion_tracking);
                    }
                    $adsConversionData = $this->apiHelper->getAdsConversionData($data->google_ads_id);

                    if (!empty($adsConversionData->data[0]->id)) {
                        $this->_configWriter->save('tatvic_ee/google_ads/conversion_id', $adsConversionData->data[0]->id);
                        $this->_configWriter->save('tatvic_ee/google_ads/conversion_lable', $adsConversionData->data[0]->name);
                    }

                    if (property_exists($adsConversionData, "error") && $adsConversionData->error == false) {
                        if (property_exists($adsConversionData, "data") && $adsConversionData->data != "" && !empty($adsConversionData->data)) {
                            foreach ($adsConversionData->data as $key => $value) {
                                $con_string = strip_tags($value->tagSnippets); //what i want is you
                                $con_string = trim(preg_replace('/\s\s+/', '', $con_string));
                                $con_string = str_replace(" ", "", $con_string);
                                $con_string = str_replace("'", "", $con_string);
                                $con_string = str_replace("return false;", "", $con_string);
                                $con_string = str_replace("event,conversion,{", ",event:conversion,", $con_string);
                                $con_array = explode(",", $con_string);
                                if (!empty($con_array) && in_array("event:conversion", $con_array)) {
                                    foreach ($con_array as $key => $con_value) {
                                        $con_val_array = explode(":", $con_value);
                                        if (in_array("send_to", $con_val_array)) {
                                            $this->_configWriter->save('tatvic_ee/google_ads/ee_conversio_send_to', $con_val_array[1]);
                                            break 2;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                if ($data->is_domain_claim == '0') {
                    $claimWebsite = $this->apiHelper->claimWebsite($postData);
                    if ($claimWebsite) {
                        if ($claimWebsite->error == 1) {
                            $data->is_domain_claim = '0';
                        } else {
                            $data->is_domain_claim = '1';
                        }
                    }
                } else {
                    $data->is_domain_claim = '1';
                }

                if ($data->tracking_option == "BOTH") {
                    $this->_configWriter->save('tatvic_ee/general/tracking_type', "both");
                }
                if ($data->tracking_option == "GA4") {
                    $this->_configWriter->save('tatvic_ee/general/tracking_type', "google_analytics_4");
                }
                if ($data->tracking_option == "UA") {
                    $this->_configWriter->save('tatvic_ee/general/tracking_type', "universal_analytics");
                }
                $linkedId = '';
                if (!empty($data->measurement_id)) {
                    $linkedId = $data->measurement_id;
                }
                if (!empty($data->property_id)) {
                    $linkedId = $data->property_id;
                }

                $pubKey = $this->cookieManager->getCookie('publicKey');
                $accessKey = '';
                if (empty($pubKey)) {
                    $accessKey = $this->getPublicAccessKey();
                } else {
                    $accessKey = $pubKey;
                }
                $linkedAds = '';
                $adsId = $data->google_ads_id;
                if (!empty($adsId)) {
                    $linkedAds = 1;
                } else {
                    $linkedAds = 0;
                }

                $this->_configWriter->save('tatvic_ee/purchase_code/enable', 1);
                $this->_configWriter->save('tatvic_ee/general/email', $emailId);
                $this->_configWriter->save('tatvic_ee/purchase_code/purchase_code', $accessKey);
                $this->_configWriter->save('tatvic_ee/general/plan_id', $data->plan_id);

                if ($data->tracking_option == "UA") {
                    $this->_configWriter->save('tatvic_ee/general/ua_id_ga', $data->property_id);
                }else
                {
                    $this->_configWriter->save('tatvic_ee/general/ua_id_ga', '');
                }
                
                if ($data->tracking_option == "GA4") {
                    $this->_configWriter->save('tatvic_ee/general/measurment_id_ga4', $data->measurement_id);
                }else
                {
                    $this->_configWriter->save('tatvic_ee/general/measurment_id_ga4', '');
                }

                if ($data->tracking_option == "BOTH") {
                    $this->_configWriter->save('tatvic_ee/general/ua_id_both', $data->property_id);
                    $this->_configWriter->save('tatvic_ee/general/measurment_id_both', $data->measurement_id);
                }else
                {
                   $this->_configWriter->save('tatvic_ee/general/ua_id_both', ''); 
                   $this->_configWriter->save('tatvic_ee/general/measurment_id_both', '');
                   // $this->_configWriter->save('tatvic_ee/general/ua_id_ga', '');
                   // $this->_configWriter->save('tatvic_ee/general/measurment_id_ga4', '');
                }

                $this->_configWriter->save('tatvic_ee/general/linked_ads_id', $data->google_ads_id);
                $this->_configWriter->save('tatvic_ee/general/linked_ua_id', $linkedId);
                $this->_configWriter->save('tatvic_ee/general/linked_merchant_id', $data->google_merchant_center_id);
                $this->_configWriter->save('tatvic_ee/general/enhanced_e_commerce_tracking', $data->enhanced_e_commerce_tracking);
                $this->_configWriter->save('tatvic_ee/general/add_gtag_snippet', $data->add_gtag_snippet);
                $this->_configWriter->save('tatvic_ee/general/merchant_id', $data->merchant_id);
                $this->_configWriter->save('tatvic_ee/general/enableClientID', $data->client_id_tracking);
                $this->_configWriter->save('tatvic_ee/general/enhance_link_attribution_tracking', $data->enhanced_link_attribution_tracking);
                $this->_configWriter->save('tatvic_ee/google_ads/linked_ads_id', $data->google_ads_id);
                $this->_configWriter->save('tatvic_ee/google_ads/linked_ua_id', $linkedId);
                $this->_configWriter->save('tatvic_ee/google_ads/linked_merchant_id', $data->google_merchant_center_id);
                $this->_configWriter->save('tatvic_ee/google_shopping/site_verified', $data->is_site_verified);
                $this->_configWriter->save('tatvic_ee/google_shopping/domain_claim', $data->is_domain_claim);
                $this->_configWriter->save('tatvic_ee/google_shopping/google_ads_linking', $linkedAds);
                $this->_configWriter->save('tatvic_ee/google_shopping/lincked_merchant_center', $data->google_merchant_center_id);
                $this->_configWriter->save('tatvic_ee/google_shopping/lincked_google_ads', $data->google_ads_id);
                $this->_configWriter->save('tatvic_ee/advance/subscription_update_date', $data->subscription_update_date);
                $this->_configWriter->save('tatvic_ee/advance/subscription_expiry_date', $data->subscription_expiry_date);
                $this->_configWriter->save('tatvic_ee/advance/grace_expiry_date', $data->grace_expiry_date);
                $this->_configWriter->save('tatvic_ee/advance/plan_name', $data->plan_name);
                $this->_configWriter->save('tatvic_ee/advance/max_limit', $data->max_limit);
                $this->_configWriter->save('tatvic_ee/advance/price', $data->price);
                $this->_configWriter->save('tatvic_ee/advance/site_url', $data->site_url);
                $this->messageManager->addSuccess(__("Configuration has been saved."));
            } else {
                $this->messageManager->addError(__($response->errors));
            }
        }
    }

    public function callMethodTag($postData, $data)
    {
        //methd using tag
        $postData['method'] = "meta";
        $siteVerificationTokenTag = $this->apiHelper->siteVerificationToken($postData);

        if (isset($siteVerificationTokenTag->data->token) && $siteVerificationTokenTag->data->token) {
            $this->apiHelper->set_ee_additional_data(array("add_site_varification_tag" => 1, "site_varification_tag_val" => base64_encode($siteVerificationTokenTag->data->token)));
            sleep(1);
            $siteVerificationTag = $this->apiHelper->siteVerification($postData);
            if (isset($siteVerificationTag->error) && !empty($siteVerificationTag->errors)) {
                $this->messageManager->addError(__($siteVerificationTag->errors[0]));
            } else {
                $data->is_site_verified = '1';
            }
        }
    }

    public function getCurrency()
    {
        $currencysymbol = $this->objectManager->get('Magento\Store\Model\StoreManagerInterface');
        return $currencysymbol->getStore()->getCurrentCurrencyCode();
    }

    public function getTimeZone()
    {
        $objDate = $this->objectManager->create('Magento\Framework\Stdlib\DateTime\Timezone');
        return $objDate->getConfigTimezone();
    }

    public function getConfirmUrl()
    {
        $adminUrl = $this->objectManager->create('Magento\Backend\Helper\Data')->getHomePageUrl();
        return $adminUrl . "system_config/index/";
    }

    public function getCurrentUrl()
    {
        /** @var \Magento\Framework\UrlInterface $urlInterface */
        $urlInterface = \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Framework\UrlInterface');
        return $urlInterface->getCurrentUrl();
    }

    public function getSubscritionId()
    {
        return $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
    }

    /**
     * Subscription data API
     */

    public function getAnalyticsData($subId)
    {
        $subscriptionId = $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
        $subsId = !empty($subId) ? $subId : $subscriptionId;
        $header = array(
            "Authorization: Bearer MTIzNA==",
            "content-type: application/json",
        );
        $curl_url = $this->_helper->getApiUrl() . "/customer-subscriptions/subscription-detail";
        $postData = [
            'subscription_id' => $subsId,
            'domain' => $this->getDomain(),
        ];
        $postData = json_encode($postData);
        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_URL => $curl_url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_TIMEOUT => 1000,
            CURLOPT_HTTPHEADER => $header,
            CURLOPT_POSTFIELDS => $postData,
        ));
        $response = curl_exec($ch);
        return json_decode($response);
    }
}
