<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\System\Config;

use Magento\Config\Block\System\Config\Form\Field;
use Magento\Backend\Block\Template\Context;
use Magento\Framework\Data\Form\Element\AbstractElement;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Framework\App\Config\ScopeConfigInterface;

class SiteVerify extends Field
{

    const XML_SITE_VERIFICATION = 'tatvic_ee/google_shopping/site_verified';

    const XML_GOOGLE_MERCHANT_ID = 'tatvic_ee/google_ads/linked_merchant_id';

    protected $_helper;

    protected $_scopeConfig;

    protected $siteVerify;

    protected $storeScope;

    /**
     *
     * @param Context $context
     * @param Data $helper
     * @param ScopeConfigInterface $_scopeConfig
     * @param array $data
     */
    public function __construct(
        Context $context,
        Data $helper,
        ScopeConfigInterface $_scopeConfig,
        array $data = []
    ) {
        $this->_helper = $helper;
        $this->_scopeConfig = $_scopeConfig;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->siteVerify = $this->_scopeConfig->getValue(self::XML_SITE_VERIFICATION, 'default', $this->storeScope);
        parent::__construct($context, $data);
    }

    /**
     * @return $this|\Magento\Config\Block\System\Config\Form\Field
     */
    protected function _prepareLayout()
    {
        parent::_prepareLayout();
        if (!$this->getTemplate()) {
            $this->setTemplate('system/config/site_verify.phtml');
        }
        return $this;
    }

    /**
     * @param AbstractElement $element
     * @return void
     */
    public function render(AbstractElement $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }
    /**
     *
     * @param AbstractElement $element
     * @return void
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * Check site is verify or not from core config data flag
     *
     * @return boolean
     */
    public function isSiteVerify()
    {
        return $this->siteVerify;
    }

    public function getCustomUrl()
    {
        return $this->getUrl('actionablegoogleanalytics/siteverify/index');
    }

    public function getButtonHtml()
    {
        $button = $this->getFailedSVG();
        return $button;
    }

    /**
     * Get Success icon png file path 
     *
     * @return void
     */
    public function getSuccessSVG()
    {
        return $this->getViewFileUrl('Conversios_GoogleAnalyticsAndShoppingIntegration::images/site-ver-success.png');
    }

    /**
     * Get failed icon png file path
     *
     * @return void
     */
    public function getFailedSVG()
    {
        return $this->getViewFileUrl('Conversios_GoogleAnalyticsAndShoppingIntegration::images/exclaimation.png');
    }

    /**
     * Get refresh loader png file path
     *
     * @return void
     */
    public function getRefreshLoader()
    {
        return $this->getViewFileUrl('Conversios_GoogleAnalyticsAndShoppingIntegration::images/refresh.png');
    }
}
