<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\System\Config;

use Magento\Config\Block\System\Config\Form\Field;
use Magento\Backend\Block\Template\Context;
use Magento\Framework\Data\Form\Element\AbstractElement;

class SyncButton extends Field
{

    /**
     * Subscription ID path of core config data
     */

    const XML_SUBSCRIPTION_ID = 'tatvic_ee/user/subscription_id';

    const XML_MERCHANT_ID = 'tatvic_ee/general/merchant_id';

    protected $_template = 'Conversios_GoogleAnalyticsAndShoppingIntegration::system/config/button.phtml';

    /**
     *
     * @param Context $context
     * @param array $data
     */
    public function __construct(Context $context, array $data = [])
    {
        parent::__construct($context, $data);
    }

    /**
     *
     * @param AbstractElement $element
     * @return void
     */
    public function render(AbstractElement $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }

    /**
     *
     * @param AbstractElement $element
     * @return void
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * Undocumented function
     *
     * @return void
     */
    public function getCustomUrl()
    {
        return $this->getUrl('actionablegoogleanalytics/product/index');
    }


    /**
     * Get subscription ID
     *
     * @return void
     */
    public function getSubscritionId()
    {
        return $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
    }


    /**
     * Get MerchantCenter ID
     *
     * @return void
     */
    public function getMerchantId()
    {
        return $this->_scopeConfig->getValue(self::XML_MERCHANT_ID, 'default', $this->storeScope);
    }


    /**
     * Undocumented function
     *
     * @return void
     */
    public function getButtonHtml()
    {
        $button = $this->getLayout()->createBlock('Magento\Backend\Block\Widget\Button')->setData(['id' => 'product_sync_btn','class' => 'secondary ui-button ui-widget ui-state-default ui-corner-all', 'label' => __('Product Sync List'),]);
        return $button->toHtml();
    }
}
