<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\System\Config;

use Magento\Backend\Block\Template\Context;
use Magento\Config\Block\System\Config\Form\Field;
use Magento\Framework\Data\Form\Element\AbstractElement;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Framework\App\Config\ScopeConfigInterface;

class SyncNewProductsButton  extends Field
{
    /**
     * Subscription ID path of core config data
     */

    const XML_SUBSCRIPTION_ID = 'tatvic_ee/user/subscription_id';

    /**
     * Define sync product button phtml file
     *
     * @var string
     */
    protected $_template = 'Conversios_GoogleAnalyticsAndShoppingIntegration::system/config/syn_new_button.phtml';

    public $_helper;

    protected $_scopeConfig;

    /**
     *
     * @param Context $context
     * @param Data $helper
     * @param ScopeConfigInterface $_scopeConfig
     * @param array $data
     */
    public function __construct(
        Context $context, 
        Data $helper,
        ScopeConfigInterface $_scopeConfig,
        array $data = []
    )
    {
        $this->_helper = $helper;
        $this->_scopeConfig = $_scopeConfig;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        parent::__construct($context, $data);
    }

    /**
     * Get subscription ID
     *
     * @return void
     */
    public function getSubscritionId()
    {
        return $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
    }

    public function render(AbstractElement $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }
    /**
     *
     * @param AbstractElement $element
     * @return void
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * get product sync phtml file url
     *
     * @return void
     */
    public function getCustomUrl()
    {
        return $this->getUrl('actionablegoogleanalytics/index/index');
    }

    /**
     * Get button html
     *
     * @return void
     */
    public function getButtonHtml()
    {
        $button = $this->getLayout()->createBlock('Magento\Backend\Block\Widget\Button')->setData(['id' => 'new_product_sync_btn','class' => 'secondary ui-button ui-widget ui-state-default ui-corner-all', 'label' => __('Sync Products'),]);
        return $button->toHtml();
    }
}
