<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Block\Adminhtml\System\Config;

use Magento\Config\Block\System\Config\Form\Field;
use Magento\Backend\Block\Template\Context;
use Magento\Framework\Data\Form\Element\AbstractElement;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\CustomApi;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Cache\Frontend\Pool;

class UpdateConfig extends Field
{
    /**
     *
     * @var string
     */
    protected $_template = 'Conversios_GoogleAnalyticsAndShoppingIntegration::system/config/update_button.phtml';

    /**
     * Get subscription ID from this config endpoint
     */
    const XML_SUBSCRIPTION_ID = 'tatvic_ee/user/subscription_id';

    /**
     * @var [$_helper]
     */
    public $_helper;

    /**
     * @var [$objectManager]
     */
    public $objectManager;

    public $connectUrl;

    protected $_configWriter;

    protected $token;

    protected $_scopeConfig;

    protected $apiDomain;

    public $storeScope;

    protected $apiHelper;

    protected $cacheTypeList;

    protected $cacheFrontendPool;

    protected $request;

    protected $cacheManager;

    protected $messageManager;

    /**
     * @param Context $context
     * @param Data $helper
     * @param CustomApi $apiHelper
     * @param WriterInterface $scopeConfig
     * @param \Magento\Framework\Serialize\Serializer\Json $json
     * @param ScopeConfigInterface $_scopeConfig
     * @param TypeListInterface $cacheTypeList
     * @param Pool $cacheFrontendPool
     * @param \Magento\Framework\App\Request\Http $request
     * @param \Magento\Framework\App\Cache\Manager $cacheManager
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param array $data
     */
    public function __construct(
        Context $context,
        Data $helper,
        CustomApi $apiHelper,
        WriterInterface $scopeConfig,
        \Magento\Framework\Serialize\Serializer\Json $json,
        ScopeConfigInterface $_scopeConfig,
        TypeListInterface $cacheTypeList,
        Pool $cacheFrontendPool,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Framework\App\Cache\Manager $cacheManager,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        array $data = []
    ) {
        $this->cacheManager = $cacheManager;
        
        $this->_configWriter = $scopeConfig;
        $this->token = 'MTIzNA==';
        $this->objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $this->_helper = $helper;
        $this->json = $json;
        $this->_scopeConfig = $_scopeConfig;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->apiHelper = $apiHelper;
        $this->cacheTypeList = $cacheTypeList;
        $this->cacheFrontendPool = $cacheFrontendPool;
        $this->messageManager = $messageManager;
        $this->request = $request;
        $this->updateCOnfiguration();
        parent::__construct($context, $data);
        
    }

    /**
     * @return $this|\Magento\Config\Block\System\Config\Form\Field
     */
    protected function _prepareLayout()
    {
        parent::_prepareLayout();
        if (!$this->getTemplate()) {
            $this->setTemplate('system/config/update_button.phtml');
        }
        return $this;
    }

    /**
     * Use for rendering the greed
     * @param AbstractElement $element
     * @return void
     */
    public function render(AbstractElement $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }

    /**
     * Get greed HTML
     * @param AbstractElement $element
     * @return void
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * Create Connect URL of preprodestore
     *
     * @return void
     */
    public function getCustomUrl()
    {
        if (!empty($this->connectUrl)) {
            return $this->connectUrl;
        } else {
            $subscriptionId = $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
            return "https://" . $this->_helper->getConnectUrl() . "/config/ga_rdr_gmc.php?return_url=" . $this->_helper->getConnectUrl() . "/config/ads-analytics-form.php?domain=" . $this->getDomain() . "&country=" . $this->getCountry() . "&user_currency=" . $this->getCurrency() . "&subscription_id=" . $subscriptionId . "&confirm_url=" . $this->getCurrentUrl() . "&timezone=".$this->getTimeZone()."&app_id=2";
        }
    }

    /**
     * Clean System Config Cache 
     *
     * @return void
     */
    public function updateCOnfiguration()
    {
        $types = array('config');
        foreach ($types as $type) {
            $this->cacheTypeList->cleanType($type);
        }
        foreach ($this->cacheFrontendPool as $cacheFrontend) {
            $cacheFrontend->getBackend()->clean();
        }
    }

    /**
     * Get Subscription Details API
     *
     * @param [type] $subId
     * @return void
     */
    public function getAnalyticsData($subId)
    {
        $subscriptionId = $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);

        $subsId = !empty($subId) ? $subId : $subscriptionId;
        $header = array(
            "Authorization: Bearer MTIzNA==",
            "content-type: application/json",
        );
        $curl_url = $this->_helper->getApiUrl() . "/customer-subscriptions/subscription-detail";
        $postData = [
            'subscription_id' => $subsId,
            'domain' => $this->getDomain(),
        ];
        $postData = json_encode($postData);
        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_URL => $curl_url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_TIMEOUT => 1000,
            CURLOPT_HTTPHEADER => $header,
            CURLOPT_POSTFIELDS => $postData,
        ));
        $response = curl_exec($ch);
        $response = json_decode($response);

        return $response;
    }

    /**
     * Get store default currency
     *
     * @return void
     */
    public function getCurrency()
    {
        $currencysymbol = $this->objectManager->get('Magento\Store\Model\StoreManagerInterface');
        return $currencyCode = $currencysymbol->getStore()->getCurrentCurrencyCode();
    }

    /**
     * Get Current store Timezone
     *
     * @return void
     */
    public function getTimeZone()
    {
        $objDate = $this->objectManager->create('Magento\Framework\Stdlib\DateTime\Timezone');
        return $objDate->getConfigTimezone();
    }

    /**
     * Get Confirm URL for redirection to store from connect page
     *
     * @return void
     */
    public function getConfirmUrl()
    {
        $adminUrl = $this->objectManager->create('Magento\Backend\Helper\Data')->getHomePageUrl();
        return $adminUrl . "system_config/index/";
    }

    /**
     * Get browser current URL
     *
     * @return void
     */
    public function getCurrentUrl()
    {
        /** @var \Magento\Framework\UrlInterface $urlInterface */
        $urlInterface = $this->objectManager->get('Magento\Framework\UrlInterface');
        return $urlInterface->getCurrentUrl();
    }

    /**
     * Get domain name
     * @return mixed
     */
    public function getDomain()
    {
        $storeManager = $this->objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        return $storeManager->getStore()->getBaseUrl();
    }

    /**
     * Get Store country
     *
     * @return void
     */
    public function getCountry()
    {
        return $this->_helper->getCountry();
    }

    /**
     * Get Button HTML for upgrade configuration from system configuration
     *
     * @return void
     */
    public function getButtonHtml()
    {
        $button = $this->getLayout()->createBlock('Magento\Backend\Block\Widget\Button')->setData(['id' => 'product_sync_btn', 'class' => 'secondary ui-button ui-widget ui-state-default ui-corner-all', 'label' => __('Update Configuration'),]);
        return $button->toHtml();
    }
}
