<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Controller\Adminhtml\RefreshProducts;

use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\CustomApi;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Serialize\SerializerInterface;
use \Magento\Framework\App\Action\Action;
use \Magento\Framework\App\Action\Context;
use \Magento\Framework\View\Result\PageFactory;
use \Psr\Log\LoggerInterface;

class Index extends Action
{

    const XML_PATH_NEXT_PAGE_TOKEN = 'tatvic/nextPageToken';
    /**
     * @var Context
     */
    protected $context;

    /**
     * @var PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var
     */
    public $storeScope;

    protected $_configWriter;

    protected $_resources;

    protected $customHelper;

    /**
     * @var Data
     */
    public $_helper;

    protected $date;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var SerializerInterface
     */
    protected $serializer;

    public function __construct(
        Context $context,
        PageFactory $pageFactory,
        JsonFactory $resultJsonFactory,
        WriterInterface $scopeConfig,
        ScopeConfigInterface $_scopeConfig,
        CustomApi $customHelper,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        LoggerInterface $logger,
        SerializerInterface $serializer,
        Data $helper
    ) {
        $this->resultPageFactory = $pageFactory;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->context = $context;
        $this->_scopeConfig = $_scopeConfig;
        $this->_configWriter = $scopeConfig;
        $this->_helper = $helper;
        $this->customHelper = $customHelper;
        $this->date = $date;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->_resources = \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Framework\App\ResourceConnection');
        $this->logger = $logger;
        $this->serializer = $serializer;
        parent::__construct($context);
    }

    public function execute($nextPage = null)
    {
        //echo "Testing Function execute";exit;
        $response = "";
        $products = "";
        $result = $this->resultJsonFactory->create();
        $connection = $this->_resources->getConnection();
        $lastRow = $this->getLastRowGmcId();
        if (!empty($lastRow)) {
            if ($lastRow[0]['gmc_id'] !== $this->_helper->getMerchantId()) {
                $connection->truncateTable('product_sync_list');
            }
        }

        if (empty($lastRow)) {
            $connection->truncateTable('product_sync_list');
        }
        
        $productData = $this->customHelper->getSyncProductList($nextPage);
        //print_r($productData);exit;
        if (!empty($productData->data)) {
            $this->logger->log(100, print_r($productData->data, true));
            $nextPage = $productData->data->nextPageToken;
            if (property_exists($productData->data, 'products')) {
                $products = $productData->data->products;
                if (isset($nextPage) && !empty($nextPage)) {
                    $this->saveProductsData($products);
                    $this->execute($nextPage);
                } else {
                    if ($productData->data->nextPageToken == '') {
                        $products = $productData->data->products;
                        $this->saveProductsData($products);
                        $this->_configWriter->save('tatvic/last_update_list', $this->serializer->serialize($this->date->gmtDate()));
                        $response = array(
                            "status" => "success",
                            "message" => "Products List Update Successfully",
                            "last_update_time" => $this->date->gmtDate(),
                        );
                        //print_r(json_encode($response));
                        $result->setData($response);
                        return $result;
                    }
                }
            } else {
                $this->_configWriter->save('tatvic/last_update_list', $this->serializer->serialize($this->date->gmtDate()));
                $response = array(
                    "status" => "failed",
                    "message" => "Products Not Found!",
                    "last_update_time" => $this->date->gmtDate(),
                );
                $connection->truncateTable('product_sync_list');
                print_r(json_encode($response));
                $result->setData($response);
                return $result;
            }
        } else {
            $this->_configWriter->save('tatvic/last_update_list', $this->serializer->serialize($this->date->gmtDate()));
            $response = array(
                "status" => "failed",
                "message" => "Products Not Found!",
                "last_update_time" => $this->date->gmtDate(),
            );
            //print_r(json_encode($response));
            $result->setData($response);
            return $result;
        }
    }

    public function getLastRowGmcId()
    {
        $connection = $this->_resources->getConnection();
        $table = $this->_resources->getTableName('product_sync_list');
        $rows = "SELECT * FROM " . $table . " WHERE gmc_id = " . (int) $this->_helper->getMerchantId() . " ORDER BY id DESC LIMIT 1";
        return $connection->fetchAll($rows);
    }

    /**
     * @param [type] $products
     * @return void
     */
    public function saveProductsData($products)
    {
        //echo "Product Save";exit;
        $connection = $this->_resources->getConnection();
        $table = $this->_resources->getTableName('product_sync_list');
        foreach ($products as $product) {

            $select = $connection->select()->distinct()->from($table, "product_id")->where("product_id = ?", $product->productId);
            $select = $connection->fetchOne($select);

            $data = [
                'gmc_id' => (int) $this->_helper->getMerchantId(),
                'imageLink' => $product->imageLink,
                'product_id' => $product->productId,
                'name' => $product->name,
                'googleStatus' => $product->googleStatus,
                'issues' => json_encode($product->issues),
            ];

            if ($select != $product->productId) {
                $connection->insertForce($table, $data);
            }

            if ($select == $product->productId) {
                $issues = json_encode(implode(",", $product->issues));
                $productId = $product->productId;
                $updateData = "UPDATE " . $table . " SET googleStatus = '" . $product->googleStatus . "', issues = " . $issues . " WHERE product_id = '" . $productId . "'";
                $connection->exec($updateData);
            }
        }
    }
}
