<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Controller\Adminhtml\domainClaim;

use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\CustomApi;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Framework\App\Cache\Frontend\Pool;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use \Magento\Framework\App\Action\Action;
use \Magento\Framework\App\Action\Context;

class Index extends Action
{
    const XML_SUBSCRIPTION_ID = 'tatvic_ee/user/subscription_id';

    const XML_IS_DOMAIN_CLAIM = 'tatvic_ee/google_shopping/domain_claim';

    const XML_GOOGLE_MERCHANT_CENTER_ID = 'tatvic_ee/general/linked_merchant_id';

    const XML_MERCHANT_ID = 'tatvic_ee/general/merchant_id';

    protected $_helper;

    protected $objectManager;

    protected $connectUrl;

    protected $_configWriter;

    protected $token;

    protected $_scopeConfig;

    protected $apiDomain;

    public $storeScope;

    protected $apiHelper;

    protected $cacheTypeList;

    protected $cacheFrontendPool;

    protected $request;

    protected $messageManager;

    protected $subscriptionId;

    protected $isDomainClaimed;

    protected $googleGerchantCenterId;

    protected $merchantId;

    /**
     * @var JsonFactory
     */
    protected $resultJsonFactory;

    public function __construct(
        Context $context,
        Data $helper,
        CustomApi $apiHelper,
        WriterInterface $scopeConfig,
        \Magento\Framework\Serialize\Serializer\Json $json,
        ScopeConfigInterface $_scopeConfig,
        TypeListInterface $cacheTypeList,
        Pool $cacheFrontendPool,
        JsonFactory $resultJsonFactory,
        \Magento\Framework\Message\ManagerInterface $managerInterface,
        \Magento\Framework\App\Request\Http $request
    ) {
        $this->_configWriter = $scopeConfig;
        $this->token = 'MTIzNA==';
        $this->objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $this->_helper = $helper;
        $this->json = $json;
        $this->_scopeConfig = $_scopeConfig;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->apiHelper = $apiHelper;
        $this->cacheTypeList = $cacheTypeList;
        $this->cacheFrontendPool = $cacheFrontendPool;
        $this->request = $request;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->subscriptionId = $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
        $this->isDomainClaimed = $this->_scopeConfig->getValue(self::XML_IS_DOMAIN_CLAIM, 'default', $this->storeScope);
        $this->googleGerchantCenterId = $this->_scopeConfig->getValue(self::XML_GOOGLE_MERCHANT_CENTER_ID, 'default', $this->storeScope);
        $this->merchantId = $this->_scopeConfig->getValue(self::XML_MERCHANT_ID, 'default', $this->storeScope);
        parent::__construct($context);
    }

    public function execute()
    {
        $result = $this->resultJsonFactory->create();

        try {
            if ($this->isDomainClaimed == 0) {
                $postData = [
                    'merchant_id' => $this->merchantId,
                    'website_url' => $this->getDomain(),
                    'subscription_id' => $this->subscriptionId,
                    'account_id' => $this->googleGerchantCenterId,
                ];

                $claimWebsite = $this->apiHelper->claimWebsite($postData);

                if ($claimWebsite) {
                    if (isset($claimWebsite->error) && !empty($claimWebsite->errors)) {
                        $result->setData($claimWebsite->errors);
                        return $result;
                    } else {
                        $this->_configWriter->save('tatvic_ee/google_shopping/domain_claim', 1);
                        $result->setData($claimWebsite);
                        return $result;
                    }
                }
            }
            $this->cacheFlushCustom();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function cacheFlushCustom()
    {
        $types = array('config', 'layout', 'block_html', 'collections', 'reflection', 'db_ddl', 'eav', 'config_integration', 'config_integration_api', 'full_page', 'translate', 'config_webservice');

        foreach ($types as $type) {
            $this->cacheTypeList->cleanType($type);
        }
        foreach ($this->cacheFrontendPool as $cacheFrontend) {
            $cacheFrontend->getBackend()->clean();
        }
    }
    /**
     * @return mixed
     */
    public function getDomain()
    {
        $storeManager = $this->objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        return $storeManager->getStore()->getBaseUrl();
    }
}
