<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Controller\Adminhtml\siteVerify;

use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\CustomApi;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Magento\Framework\App\Cache\Frontend\Pool;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use \Magento\Framework\App\Action\Action;
use \Magento\Framework\App\Action\Context;

class Index extends Action
{

    const XML_SUBSCRIPTION_ID = 'tatvic_ee/user/subscription_id';

    const XML_IS_SITE_VERIFIED = 'tatvic_ee/google_shopping/site_verified';

    const XML_GOOGLE_MERCHANT_CENTER_ID = 'tatvic_ee/general/linked_merchant_id';

    const XML_MERCHANT_ID = 'tatvic_ee/general/merchant_id';

    protected $_helper;

    protected $objectManager;

    protected $connectUrl;

    protected $_configWriter;

    protected $token;

    protected $_scopeConfig;

    protected $apiDomain;

    protected $storeScope;

    protected $apiHelper;

    protected $cacheTypeList;

    protected $cacheFrontendPool;

    protected $request;

    protected $messageManager;

    protected $subscriptionId;

    protected $isSiteVerified;

    protected $googleGerchantCenterId;

    protected $merchantId;

    /**
     * @var JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * Site Verify Class contruct
     *
     * @param Context $context
     * @param Data $helper
     * @param CustomApi $apiHelper
     * @param WriterInterface $scopeConfig
     * @param \Magento\Framework\Serialize\Serializer\Json $json
     * @param ScopeConfigInterface $_scopeConfig
     * @param TypeListInterface $cacheTypeList
     * @param Pool $cacheFrontendPool
     * @param JsonFactory $resultJsonFactory
     * @param \Magento\Framework\Message\ManagerInterface $managerInterface
     * @param \Magento\Framework\App\Request\Http $request
     */
    public function __construct(
        Context $context,
        Data $helper,
        CustomApi $apiHelper,
        WriterInterface $scopeConfig,
        \Magento\Framework\Serialize\Serializer\Json $json,
        ScopeConfigInterface $_scopeConfig,
        TypeListInterface $cacheTypeList,
        Pool $cacheFrontendPool,
        JsonFactory $resultJsonFactory,
        \Magento\Framework\Message\ManagerInterface $managerInterface,
        \Magento\Framework\App\Request\Http $request
    ) {
        $this->_configWriter = $scopeConfig;
        $this->token = 'MTIzNA==';
        $this->objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $this->_helper = $helper;
        $this->json = $json;
        $this->_scopeConfig = $_scopeConfig;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->apiHelper = $apiHelper;
        $this->cacheTypeList = $cacheTypeList;
        $this->cacheFrontendPool = $cacheFrontendPool;
        $this->request = $request;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->subscriptionId = $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
        $this->isSiteVerified = $this->_scopeConfig->getValue(self::XML_IS_SITE_VERIFIED, 'default', $this->storeScope);
        $this->googleGerchantCenterId = $this->_scopeConfig->getValue(self::XML_GOOGLE_MERCHANT_CENTER_ID, 'default', $this->storeScope);
        $this->merchantId = $this->_scopeConfig->getValue(self::XML_MERCHANT_ID, 'default', $this->storeScope);
        $this->messageManager = $managerInterface;
        parent::__construct($context);
    }

    public function execute()
    {
        $result = $this->resultJsonFactory->create();
        if ($this->isSiteVerified == 0) {
            $response = $this->apiHelper->callSiteVarification();
            $result->setData($response);
            return $result;
        }
    }

    /**
     * @return mixed
     */
    public function getDomain()
    {
        $storeManager = $this->objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        return $storeManager->getStore()->getBaseUrl();
    }
}
