<?php
/**
 * Conversios Software
 *
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */
namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Cron;

use Magento\Framework\App\ResourceConnection;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Helper\Data;
use Conversios\GoogleAnalyticsAndShoppingIntegration\Controller\Adminhtml\productsSync\Index;
use Magento\Framework\Serialize\SerializerInterface;

class AutoSyncModel extends Index
{
    protected $_resources;

    protected $connection;

    protected $syncDataTable;

    protected $helper;

    protected $logger;

    protected $writer;

    protected $syncIndex;

    /**
     * Auto Sync Cron class
     *
     * @param ResourceConnection $resources
     * @param Data $helper
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        ResourceConnection $resources,
        Index $syncIndex,
        Data $helper
    ) {
        $this->_resources = $resources;
        $this->connection = $this->_resources->getConnection();
        $this->syncDataTable = $this->_resources->getTableName('`product_sync_data`');
        $this->helper = $helper;
        $this->syncIndex = $syncIndex;
    }

    public function productsAutoSync()
    {
        $this->logger = new \Zend\Log\Logger();
        $this->writer = new \Zend\Log\Writer\Stream(BP . '/var/log/product_sync.log');
        $this->logger->addWriter($this->writer);
        try {
            $curl_url = $this->helper->getApiUrl() . "/products/batch";
            $batchId = time();
            //your cron job code
            $data = $this->getAutoSyncProducts();
            $currency = $this->helper->getCurrencyCode();
            $entries = [];
            if (!empty($data)) {
                foreach ($data as $productData) {
                    $postDatas = json_decode($productData['attribute_mapping']);
                    $product = $this->syncIndex->getLoadProduct($productData['product_id']);
                    if ($product->getTypeId() == 'configurable') {
                        $_children = $product->getTypeInstance(true)->getUsedProducts($product);
                        if ($_children) {
                            foreach ($_children as $child) {
                                $childProduct = $this->syncIndex->getLoadProduct($child->getID());
                                $prodData = $childProduct->getData();
                                $id = ($childProduct->getId() != '') ? $childProduct->getId() : $childProduct->getSku();
                                $helperImport = $this->syncIndex->objectManager->get('\Magento\Catalog\Helper\Image');
                                $productsData['content_language'] = !empty($postDatas->content_language) ? $postDatas->content_language : "en";
                                $productsData['target_country'] = !empty($postDatas->target_country) ? $postDatas->target_country : $this->syncIndex->getCountry();
                                $productsData['condition'] = !empty($postDatas->condition) ? $postDatas->condition : "new";
                                $productsData['offer_id'] = (int) $id;
                                $productsData['channel'] = "online";
                                $productsData['item_group_id'] = (int) $product->getId();
                                $productsData['google_product_category'] = (int) $productData['gmc_cat_id'];
                                $productsData['link'] = $this->syncIndex->objectManager->get('Magento\Store\Model\StoreManagerInterface')->getStore()->getBaseUrl() . $product->getUrlKey() . '.html';
                                $productsData['image_link'] = (string) $helperImport->init($childProduct, 'product_page_image_small')->setImageFile($childProduct->getSmallImage())->resize(380)->getUrl();
                                $productsData['id'] = (int) $id;
                                $productsData['title'] = $childProduct->getName();
                                $productsData['description'] = (string) strip_tags($childProduct->getData('description'));
                                $productsData['brand'] = !empty($postDatas->brand) ? (!empty($childProduct->getAttributeText('manufacturer')) ? $childProduct->getAttributeText('manufacturer') : "") : $childProduct->getAttributeText('manufacturer');
                                $productsData['age_group'] = !empty($postDatas->age_group) ? $postDatas->age_group : 'adult';
                                $productsData['gender'] = !empty($postDatas->gender) ? $postDatas->gender : 'male';
                                $productsData['availability'] = ($childProduct->isInStock() == 1) ? "in stock" : "out of stock";
                                $productsData['gtin'] = isset($postDatas->gtin) ? $childProduct->getResource()->getAttribute($postDatas->gtin) : '';
                                $productsData['mpn'] = isset($postDatas->mpn) ? $childProduct->getResource()->getAttribute($postDatas->mpn) : '';
                                $productsData['price'] = ["value" => $childProduct->getPrice(), "currency" => $currency];
                                $productsData['sale_price'] = ["value" => $childProduct->getFinalPrice(), "currency" => $currency];
                                $attribute = $childProduct->getResource()->getAttribute('size');
                                $attributeValue = $attribute->getFrontend()->getValue($childProduct);
                                $attribute2 = $childProduct->getResource()->getAttribute('color');
                                $attributeValue2 = $attribute2->getFrontend()->getValue($childProduct);
                                $sizes = !empty($postDatas->sizes) ? (!empty($attributeValue) ? $attributeValue : "") : "";
                                $color = !empty($postDatas->color) ? (!empty($attributeValue2) ? $attributeValue2 : "") : "";
                                $productsData['color'] = $color;
                                $productsData['sizes'] = $sizes;
                                $productsData['taxes'] = array(
                                    "rate" => (int) !empty($postDatas->tax) ? $postDatas->tax : '',
                                    "country" => !empty($postDatas->target_country) ? $postDatas->target_country : 'US'
                                );
                                $productsData['shipping'] = array(
                                    "price" => ([
                                        "value" => (int) !empty($postDatas->shipping) ? $postDatas->shipping : '',
                                        "currency" => $currency
                                    ]),
                                    "country" => !empty($postDatas->target_country) ? $postDatas->target_country : 'US'
                                );
                                $entrie = [
                                    'method' => 'insert',
                                    'merchant_id' => $this->syncIndex->getMerchantId(),
                                    'batch_id' => ++$batchId,
                                    'product' => $productsData,
                                ];
                                $entries[] = $entrie;
                            }
                        } else {
                            do {
                                $prodData = $product->getData();
                                $id = ($prodData['entity_id'] != '') ? $prodData['entity_id'] : $prodData['sku'];
                                $helperImport = $this->syncIndex->objectManager->get('\Magento\Catalog\Helper\Image');
                                $productsData['content_language'] = !empty($postDatas->content_language) ? $postDatas->content_language : "en";
                                $productsData['target_country'] = !empty($postDatas->target_country) ? $postDatas->target_country : $this->syncIndex->getCountry();
                                $productsData['condition'] = !empty($postDatas->condition) ? $postDatas->condition : "new";
                                $productsData['offer_id'] = (int) $id;
                                $productsData['channel'] = "online";
                                $productsData['google_product_category'] = (int) $productData['gmc_cat_id'];
                                $productsData['link'] = $this->syncIndex->objectManager->get('Magento\Store\Model\StoreManagerInterface')->getStore()->getBaseUrl() . $product->getUrlKey() . '.html';
                                $productsData['image_link'] = (string) $helperImport->init($product, 'product_page_image_small')->setImageFile($product->getSmallImage())->resize(380)->getUrl();
                                $productsData['id'] = (int) $id;
                                $productsData['title'] = $prodData['name'];
                                $productsData['description'] = (string) strip_tags($prodData['description']);
                                $productsData['brand'] = !empty($postDatas->brand) ? (!empty($product->getAttributeText('manufacturer')) ? $product->getAttributeText('manufacturer') : "") : $product->getAttributeText('manufacturer');
                                $productsData['age_group'] = !empty($postDatas->age_group) ? $postDatas->age_group : 'adult';
                                $productsData['gender'] = !empty($postDatas->gender) ? $postDatas->gender : 'male';
                                $productsData['availability'] = ($product->isInStock() == 1) ? "in stock" : "out of stock";
                                $productsData['gtin'] = !empty($postDatas->gtin) ? $childProduct->getResource()->getAttribute($postDatas->gtin) : '';
                                $productsData['mpn'] = !empty($postDatas->mpn) ? $childProduct->getResource()->getAttribute($postDatas->mpn) : '';
                                $productsData['taxes'] = array(
                                    "rate" => !empty($postDatas->tax) ? $postDatas->tax : '',
                                    "country" => !empty($postDatas->target_country) ? $postDatas->target_country : 'US'
                                );
                                $productsData['shipping'] = array(
                                    "price" => ([
                                        "value" => (int) !empty($postDatas->shipping) ? $postDatas->shipping : '',
                                        "currency" => $currency
                                    ]),
                                    "country" => !empty($postDatas->target_country) ? $postDatas->target_country : 'US'
                                );

                                $attribute = $product->getResource()->getAttribute('size');
                                $attributeValue = $attribute->getFrontend()->getValue($product);
                                $attribute2 = $product->getResource()->getAttribute('color');
                                $attributeValue2 = $attribute2->getFrontend()->getValue($product);
                                $sizes = !empty($postDatas->sizes) ? (!empty($attributeValue) ? $attributeValue : "") : "";
                                $color = !empty($postDatas->color) ? (!empty($attributeValue2) ? $attributeValue2 : "") : "";

                                if ($product->getFinalPrice() > 0) {
                                    $price = !empty($product->getFinalPrice()) ? $product->getFinalPrice() : $prodData->price;
                                    $productsData['price'] = ["value" => $price, "currency" => $currency];
                                }

                                $productsData['link'] = $this->syncIndex->objectManager->get('Magento\Store\Model\StoreManagerInterface')->getStore()->getBaseUrl() . $product->getUrlKey() . '.html';
                                if ($product->getSpecialPrice() > 0) {
                                    $specialPrice = !empty($product->getSpecialPrice()) ? $product->getSpecialPrice() : $prodData['price'];
                                    $productsData['sale_price'] = ["value" => number_format($specialPrice, 2), "currency" => $currency];
                                }

                                $productsData['color'] = $color;
                                $productsData['sizes'] = $sizes;

                                $entrie = [
                                    'method' => 'insert',
                                    'merchant_id' => $this->syncIndex->getMerchantId(),
                                    'batch_id' => ++$batchId,
                                    'product' => $productsData,
                                ];
                                $entries[] = $entrie;
                            } while ($product->getTypeId() == 'simple');
                        }
                    }
                    $datas = [
                        'subscription_id' => $this->syncIndex->getSubscriptionId(),
                        'merchant_id' => $this->syncIndex->getAccountId(),
                        'account_id' => $this->syncIndex->getMerchantId(),
                        'entries' => $entries,
                    ];

                    $postData = json_encode($datas);

                    $header = [
                        "Authorization:Bearer MTIzNA==",
                        "content-type:application/json",
                        "AccessToken:" . $this->syncIndex->generateAccessToken($this->syncIndex->get_tvc_access_token(), $this->syncIndex->get_tvc_refresh_token()),
                    ];

                    $ch = curl_init();

                    curl_setopt($ch, CURLOPT_URL, $curl_url);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                    curl_setopt($ch, CURLOPT_POST, true);
                    curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
                    curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);
                    curl_setopt($ch, CURLOPT_TIMEOUT, 10000);

                    $server_output = curl_exec($ch);

                    $sync_produt = json_decode($server_output);
                    if ($sync_produt->error) {
                        $response = array('status' => 'success', "syncProducts" => $sync_produt->errors);
                        $this->logger->log(7, print_r($response, true));
                    } else {
                        $sync_produts = count($sync_produt->data->entries);
                        $response = array('status' => 'success', "syncProducts" => $sync_produts);
                        $this->logger->log(7, print_r($response, true));
                        $updateDate = date('Y-m-d', strtotime(' +25 day'));
                        $this->updateSyncDate($productData['product_id'], $updateDate);
                        $this->syncIndex->_configWriter->save('tatvic/last_update_list', $this->serializer->serialize($this->date->gmtDate()));
                    }
                }
            }
        } catch (Exception $e) {
            $this->logger->info('An error has occured');
        }
    }

    public function getAutoSyncProducts()
    {
        $todayDate = date('Y-m-d');
        $products = "SELECT * FROM " . $this->syncDataTable . " WHERE update_date = '" . $todayDate . "'";
        return $this->connection->fetchAll($products);
    }

    public function updateSyncDate($productId, $updateDate)
    {
        $updateData = "UPDATE " . $this->syncDataTable . " SET update_date = " . $updateDate . " WHERE product_id = " . $productId;
        $this->connection->query($updateData);
    }
}
