<?php
/**
 * Conversios Software
 * @category Conversios
 * @package Conversios_GoogleAnalyticsAndShoppingIntegration
 * @author Conversios
 * @copyright Copyright (c)  Conversios Analytics LLC (https://conversios.io/)
 */

namespace Conversios\GoogleAnalyticsAndShoppingIntegration\Helper;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\Serialize\SerializerInterface;
use PHPUnit\Framework\Exception;

class CustomApi
{
    protected $maxResults = 80;

    protected $apiDomain;

    protected $token;

    protected $access_token;

    protected $refresh_token;

    protected $storeScope;

    protected $_scopeConfig;

    protected $configWriter;

    protected $apiHelper;

    const XML_ACCESS_TOKEN = 'tatvic_ee/advance/access_token';

    const XML_REFRESH_TOKEN = 'tatvic_ee/advance/refresh_token';

    const XML_SITE_VERIFIED = 'tatvic_ee/google_shopping/site_verified';

    const XML_MERCHANT_ID = 'tatvic_ee/general/merchant_id';

    const XML_USER_MERCHANT_ID = 'tatvic_ee/general/linked_merchant_id';

    const XML_SITE_URL = 'tatvic_ee/advance/site_url';

    const XML_SUBSCRIPTION_ID = 'tatvic_ee/user/subscription_id';

    const XML_ACCOUNT_ID = 'tatvic_ee/user/subscription_id';

    const XML_ADDITIONAL_DATA = 'tatvic_ee/general/additional_data';

    const XML_CUSTOMER_ID = 'tatvic_ee/general/linked_ads_id' ?? 'tatvic_ee/google_ads/linked_ads_id';

    protected $response;

    public $objectManager;

    public $merchantAttributes;

    /**
     * @var SerializerInterface
     */
    protected $serializer;

    protected $curl;

    /**
     * Custom API constructor
     *
     * @param ScopeConfigInterface $_scopeConfig
     * @param WriterInterface $scopeConfig
     */
    public function __construct(
        ScopeConfigInterface $_scopeConfig,
        SerializerInterface $serializer,
        \Magento\Framework\HTTP\Client\Curl $curl,
        WriterInterface $scopeConfig
    ) {
        $this->_scopeConfig = $_scopeConfig;
        $this->_configWriter = $scopeConfig;
        $this->storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->apiDomain = 'https://connect.tatvic.com/laravelapi/public/api';
        $this->objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $this->serializer = $serializer;
        $this->curl = $curl;
        $this->token = 'MTIzNA==';
    }
    /**
     * @param $access_token
     * @param $refresh_token
     * @return mixed
     */

    public function getMerchantAttributes()
    {
        $this->merchantAttributes = __DIR__ . '/Json/gmc_attrbutes.json';
        $attributes = file_get_contents($this->merchantAttributes);
        return $attributes ? json_decode($attributes, true) : [];
    }
    public function getCountries()
    {
        $countries = __DIR__ . '/Json/countries.json';
        $country = file_get_contents($countries);
        return $country ? json_decode($country, true) : [];
    }

    public function getContentLanguage()
    {
        $language = __DIR__ . '/Json/iso_lang.json';
        $languages = file_get_contents($language);
        return $languages ? json_decode($languages, true) : [];
    }

    /**
     * Getnerate access token API
     *
     * @param [type] $access_token
     * @param [type] $refresh_token
     * @return void
     */
    public function generateAccessToken($access_token, $refresh_token)
    {
        $request = "https://www.googleapis.com/oauth2/v1/tokeninfo?access_token=" . $access_token;
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $request);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        $response = curl_exec($ch);
        $result = json_decode($response);

        if (isset($result->error) && $result->error) {
            $credentials_file = __DIR__ . '/Json/client-secrets.json';

            $str = file_get_contents($credentials_file);
            $credentials = $str ? json_decode($str, true) : [];
            $url = 'https://www.googleapis.com/oauth2/v4/token';
            $header = array("content-type: application/json");
            $clientId = $credentials['web']['client_id'];
            $clientSecret = $credentials['web']['client_secret'];
            $refreshToken = $refresh_token;
            $data = [
                "grant_type" => 'refresh_token',
                "client_id" => $clientId,
                'client_secret' => $clientSecret,
                'refresh_token' => $refreshToken,
            ];

            $postData = json_encode($data);
            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_HTTPHEADER => $header,
                CURLOPT_POSTFIELDS => $postData,
            ));
            $response = curl_exec($ch);

            $response = json_decode($response);

            if (isset($response->access_token)) {
                return $response->access_token;
            } else {
                return $access_token;
            }
        } else {
            return $access_token;
        }
    }

    /**
     * Get access token
     *
     * @return void
     */
    public function getAccessToken()
    {
        if (!empty($this->access_token)) {
            return $this->access_token;
        } else {
            return $this->_scopeConfig->getValue(self::XML_ACCESS_TOKEN, 'default', $this->storeScope);
        }
    }

    /**
     * Get refresh token
     *
     * @return void
     */
    public function getRefreshTOken()
    {
        if (!empty($this->refresh_token)) {
            return $this->refresh_token;
        } else {
            return $this->_scopeConfig->getValue(self::XML_REFRESH_TOKEN, 'default', $this->storeScope);
        }
    }

    /**
     * @param $postData
     * @return mixed|string
     */

    public function siteVerificationToken($postData)
    {
        try {
            $url = $this->apiDomain . '/gmc/site-verification-token';
            $this->header = array(
                "Authorization: Bearer MTIzNA==",
                "content-type: application/json",
                "AccessToken:" . $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken()),
            );

            $data = [
                'merchant_id' => $postData['merchant_id'],
                'account_id' => $postData['account_id'],
                'website' => $postData['website_url'],
                'method' => $postData['method'],
            ];

            $data = json_encode($data);
            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 10000,
                CURLOPT_HTTPHEADER => $this->header,
                CURLOPT_POSTFIELDS => $data,
            ));
            $this->response = curl_exec($ch);
            return json_decode($this->response);
        } catch (Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    /**
     * Manual site verification API
     *
     * @param [type] $postData
     * @return void
     */
    public function siteVerification($postData)
    {
        try {
            $url = $this->apiDomain . '/gmc/site-verification';
            $this->header = array(
                "Authorization: Bearer MTIzNA==",
                "content-type: application/json",
                "AccessToken:" . $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken()),
            );

            $data = [
                'merchant_id' => $postData['merchant_id'],
                'website' => $postData['website_url'],
                'subscription_id' => $postData['subscription_id'],
                'account_id' => $postData['account_id'],
                'access_token' => $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken()),
                'method' => $postData['method'],
            ];

            $data = json_encode($data);
            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 10000,
                CURLOPT_HTTPHEADER => $this->header,
                CURLOPT_POSTFIELDS => $data,
            ));
            $this->response = curl_exec($ch);
            return json_decode($this->response);
        } catch (Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    /**
     * Manual Domain claim API
     */
    public function claimWebsite($postData)
    {
        try {
            $url = $this->apiDomain . '/gmc/claim-website';
            $this->header = array(
                "Authorization:Bearer MTIzNA==",
                "content-type:application/json",
                "AccessToken:" . $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken()),
            );

            $data = [
                'merchant_id' => $postData['merchant_id'],
                'account_id' => $postData['account_id'],
                'website' => $postData['website_url'],
                'access_token' => $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken()),
                'subscription_id' => $postData['subscription_id'],
            ];

            $data = json_encode($data);
            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $url, //esc_url($this->curl_url),
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 10000,
                CURLOPT_HTTPHEADER => $this->header,
                CURLOPT_POSTFIELDS => $data,
            ));
            $this->response = curl_exec($ch);
            return json_decode($this->response);
        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    /**
     * Site verification call for manual
     *
     * @return void
     */
    public function callSiteVarification()
    {
        $directory = $this->objectManager->get('\Magento\Framework\Filesystem\DirectoryList');
        $siteVarified = $this->_scopeConfig->getValue(self::XML_SITE_VERIFIED, 'default', $this->storeScope);
        $merchantId = $this->_scopeConfig->getValue(self::XML_MERCHANT_ID, 'default', $this->storeScope);
        $subscriptionId = $this->_scopeConfig->getValue(self::XML_SUBSCRIPTION_ID, 'default', $this->storeScope);
        $accountId = $this->_scopeConfig->getValue(self::XML_USER_MERCHANT_ID, 'default', $this->storeScope);
        $siteUrl = $this->_scopeConfig->getValue(self::XML_SITE_URL, 'default', $this->storeScope);
        $siteUrl = !empty($siteUrl) ? $siteUrl : $this->getDomain();
        if ($siteVarified == 0) {
            $postData = [
                'merchant_id' => $merchantId,
                'website_url' => $siteUrl,
                'subscription_id' => $subscriptionId,
                'account_id' => $accountId,
            ];
            $postData['method'] = "file";
            $siteVerificationToken = $this->siteVerificationToken($postData);

            if (isset($siteVerificationToken->error) && !empty($siteVerificationToken->errors)) {
                do {
                    $this->callMethodTagApi();
                } while ($siteVerificationToken->error);
            } else {
                $myFile = $directory->getRoot() . "/" . $siteVerificationToken->data->token;
                if (!file_exists($myFile)) {
                    $fh = fopen($myFile, 'w+');
                    chmod($myFile, 0777);
                    $stringData = "google-site-verification: " . $siteVerificationToken->data->token;
                    fwrite($fh, $stringData);
                    fclose($fh);
                }
                $postData['method'] = "file";
                $siteVerification = $this->siteVerification($postData);
                if (isset($siteVerification->error) && !empty($siteVerification->errors)) {
                    return array('error' => true, 'msg' => $siteVerification->errors[0]);
                } elseif ($siteVerification->data->site->identifier !== $this->getDomain()) {
                    $this->_configWriter->save(self::XML_SITE_VERIFIED, 0);
                    return array('error' => true, 'msg' => 'Merchant center account already verified with ' . $siteVerification->data->site->identifier);
                } else {
                    $this->_configWriter->save(self::XML_SITE_VERIFIED, 1);
                    return array('error' => false, 'msg' => "Site verification successfully.");
                }
            }
        } else {
            return array('error' => false, 'msg' => "Already site verification successfully.");
        }
    }

    public function callMethodTagApi()
    {
        //methd using tag
        $postData['method'] = "meta";
        $siteVerificationToken_tag = $this->siteVerificationToken($postData);
        if (isset($siteVerificationToken_tag->data->token) && $siteVerificationToken_tag->data->token) {
            $ee_additional_data = $this->get_ee_additional_data();
            $ee_additional_data['add_site_varification_tag'] = 1;
            $ee_additional_data['site_varification_tag_val'] = base64_encode($siteVerificationToken_tag->data->token);

            $this->set_ee_additional_data($ee_additional_data);
            sleep(1);
            $siteVerification_tag = $this->siteVerification($postData);
            if (isset($siteVerification_tag->error) && !empty($siteVerification_tag->errors)) {
                return array('error' => true, 'msg' => $siteVerification_tag->errors[0]);
            } else {
                $this->_configWriter->save(self::XML_SITE_VERIFIED, 1);
                return array('error' => false, 'msg' => "Site Verification Successfully.");
            }
        } else {
            if (!empty($siteVerificationToken_tag->errors->merchant_id)) {
                return array('error' => true, 'msg' => $siteVerificationToken_tag->errors);
            } elseif (!empty($siteVerificationToken_tag->errors[0])) {
                return array('error' => true, 'msg' => $siteVerificationToken_tag->errors[0]);
            } else {
                return array('error' => true, 'msg' => $siteVerificationToken_tag->errors[0]);
            }
        }
        // one more try
    }

    public function createShippingCampaign($data1)
    {

        $merchantId = $this->_scopeConfig->getValue(self::XML_USER_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);
        try {
            $url = $this->apiDomain . '/campaigns/create';
            $data = [
                'merchant_id' => $merchantId,
                'customer_id' => $customerId,
                'campaign_name' => $data1['campaign_name'],
                'budget' => $data1['campaign_budget'],
                'target_country' => $data1['target_country'],
                'all_products' => $data1['all_products'],
                'filter_by' => 'category',
                'filter_data' => ["id" => '', "level" => ''],
            ];

            // print_r($data);exit;

            // post method
            $this->curl->post($url, $data);
            $headers = ["Content-Type" => "application/json", "Authorization" => "Bearer {$this->token}"];
            $this->curl->setHeaders($headers);
            $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);
            
            // output of curl requestt
            return $this->curl->getBody();

        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    public function updateShoppingCampaign($datas)
    {
        $merchantId = $this->_scopeConfig->getValue(self::XML_USER_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);

        try {

            $url = $this->apiDomain . '/campaigns/update';
            $data = [
                'merchant_id' => $merchantId,
                'customer_id' => $customerId,
                'campaign_id' => $datas['campaign_id'],
                'account_budget_id' => $datas['account_budget_id'],
                'campaign_name' => $datas['campaign_name'],
                'target_country' => $datas['target_country'],
                'budget' => $datas['budget'],
                'status' => 2, // ENABLE => 2, PAUSED => 3, REMOVED => 4
                'all_products' => 1,
                'ad_group_id' => $datas['ad_group_id'],
                'ad_group_resource_name' => $datas['ad_group_resource_name'],
                'filter_by' => 'category',
                'filter_data' => json_encode(['id' => '', 'level' => ''])
            ];
            
            $this->header = array(
                "Authorization: Bearer MTIzNA==",
                "Content-Type: application/json"
            );

            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $url, //esc_url($this->curl_url),
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 10000,
                CURLOPT_HTTPHEADER => $this->header,
                CURLOPT_POSTFIELDS => json_encode($data),
                CURLOPT_CUSTOMREQUEST => 'PATCH',
            ));
            $response = curl_exec($ch);
            // print_r($data);
            // exit;
            return json_decode($response);

        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    public function listShippingCampaign()
    {

        $merchantId = $this->_scopeConfig->getValue(self::XML_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);
        try {
            $url = $this->apiDomain . '/campaigns/list';
            $data = [
                'merchant_id' => $merchantId,
                'customer_id' => $customerId,
            ];

            // post method
            $this->curl->post($url, $data);
            $this->header = array(
                "Authorization:Bearer MTIzNA==",
                "content-type:application/json",
                "AccessToken:" . $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken()),
            );
            $this->curl->setHeaders($this->header);
            $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);

            // output of curl requestt
            return $this->curl->getBody();

        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    /**
     * Get sync products link API
     */

    public function getSyncProductList($nextPage)
    {
        $merchant = $this->_scopeConfig->getValue(self::XML_USER_MERCHANT_ID, 'default', $this->storeScope);
        try {
            $postData["maxResults"] = $this->maxResults;
            $postData['merchant_id'] = $merchant;
            if ($nextPage != "") {
                $postData['pageToken'] = $nextPage;
            } else {
                $postData['pageToken'] = '';
            }
            $header = array(
                "Authorization:Bearer MTIzNA==",
                "content-type:application/json",
                "AccessToken:" . $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken()),
            );

            $url = $this->apiDomain . '/products/list';
            $postData = json_encode($postData);
            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 1000000,
                CURLOPT_HTTPHEADER => $header,
                CURLOPT_POSTFIELDS => $postData,
            ));
            $response = curl_exec($ch);
            $response = json_decode($response);

            $return = new \stdClass();
            if (isset($response->error) && $response->error == '') {
                $return->status = 200;
                $return->data = $response->data;
                $return->message = $response->message;
                return $return;
            } else {
                if (isset($response->data)) {
                    $return->data = $response->data;
                    $return->message = $response->message;
                } else {
                    $return->data = [];
                    $return->message = '';
                }
                return $return;
            }
        } catch (\Exception $e) {
            return $e->getMessage();
        }
    }

    public function getAdsConversionData($customer_id)
    {
        try {
            $header = array(
                "Authorization: Bearer MTIzNA==",
                "content-type: application/json",
            );
            $curl_url = $this->apiDomain . "/google-ads/conversion-list";
            $postData = [
                'customer_id' => $customer_id,
            ];
            $postData = json_encode($postData);
            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $curl_url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 1000,
                CURLOPT_HTTPHEADER => $header,
                CURLOPT_POSTFIELDS => $postData,
            ));
            $response = curl_exec($ch);
            $response = json_decode($response);

            $return = new \stdClass();
            if (isset($response->error) && $response->error == '') {
                $return->error = false;
                $return->data = $response->data;
                $return->message = $response->message;
                return $return;
            } else {
                if (isset($response->data)) {
                    $return->error = false;
                    $return->data = $response->data;
                    $return->message = $response->message;
                } else {
                    $return->error = true;
                    $return->data = [];
                    if (isset($response->errors->key[0])) {
                        $return->message = $response->errors->key[0];
                    } else {
                        $return->message = "";
                    }
                }
                return $return;
            }
        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    public function getRemarketingSnippets($customer_id)
    {
        try {
            $header = array(
                "Authorization: Bearer MTIzNA==",
                "content-type: application/json",
            );
            $curl_url = $this->apiDomain . "/google-ads/remarketing-snippets";
            $postData = [
                'customer_id' => $customer_id,
            ];
            $postData = json_encode($postData);
            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $curl_url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 1000,
                CURLOPT_HTTPHEADER => $header,
                CURLOPT_POSTFIELDS => $postData,
            ));
            $response = curl_exec($ch);
            $response = json_decode($response);
            $return = new \stdClass();
            if (isset($response->error) && $response->error == '') {
                $return->error = false;
                $return->data = $response->data;
                $return->message = $response->message;
                return $return;
            } else {
                if (isset($response->data)) {
                    $return->error = false;
                    $return->data = $response->data;
                    $return->message = $response->message;
                } else {
                    $return->error = true;
                    $return->data = [];
                    if (isset($response->errors->key[0])) {
                        $return->message = $response->errors->key[0];
                    } else {
                        $return->message = "";
                    }
                }
                return $return;
            }
        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    public function set_ee_options_data($ee_options_data)
    {
        $this->_configWriter->save('tatvic_ee/general/option_data', $this->serializer->serialize($ee_options_data));
    }

    public function set_ee_additional_data($ee_additional_data)
    {
        $this->_configWriter->save('tatvic_ee/general/additional_data', $this->serializer->serialize($ee_additional_data));
    }

    public function get_ee_additional_data()
    {
        return $this->serializer->unserialize($this->_scopeConfig->getValue(self::XML_ADDITIONAL_DATA, 'default', $this->storeScope));
    }

    /**
     * @return mixed
     */
    public function getDomain()
    {
        $storeManager = $this->objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        return $storeManager->getStore()->getBaseUrl();
    }

    public function getCampaignDetails($cmpID)
    {
        $merchantId = $this->_scopeConfig->getValue(self::XML_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);
        // print_r($datas);exit;
        try {
            $url = $this->apiDomain . '/campaigns/detail';
            $data = [
                'merchant_id' => $merchantId,
                'customer_id' => $customerId,
                'campaign_id' => $cmpID
            ];
            $postData = json_encode($data);
            // print_r($data);exit;
            // post method
            $this->curl->post($url, $data);
            $headers = ["Content-Type" => "application/json", "Authorization" => "Bearer {$this->token}"];
            $this->curl->setHeaders($headers);
            $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);
            
            // output of curl requestt
            return $this->curl->getBody();

        } catch (Exception $e) {
            return $e->getMessage();
        }

    }

    public function accountPerformance($args)
    {
        
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);
        $url = $this->apiDomain . '/reports/account-performance';
        $data = [
            'customer_id' => $customerId,
            'from_date' => $args['from_date'],
            'to_date' => $args['to_date'],
            'customRadio' => 2
        ];

        $this->header = array(
            "Authorization: Bearer MTIzNA==",
            "content-type: application/json",
        );

        $response_body = json_encode($data);
        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_URL => $url, //esc_url($this->curl_url),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_TIMEOUT => 1000,
            CURLOPT_HTTPHEADER => $this->header,
            CURLOPT_POSTFIELDS => $response_body,
        ));
        $this->response = curl_exec($ch);
        return json_decode($this->response);

    }

    public function deleteShoppingCampaign($datas)
    {
        $merchantId = $this->_scopeConfig->getValue(self::XML_USER_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);

        try {
            $url = $this->apiDomain . '/campaigns/delete';
            $data = [
                'merchant_id' => $merchantId,
                'customer_id' => $customerId,
                'campaign_id' => $datas['campaign_id'],
            ];
            
            $this->header = array(
                "Authorization: Bearer MTIzNA==",
                "Content-Type: application/json"
            );

            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $url, //esc_url($this->curl_url),
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 10000,
                CURLOPT_HTTPHEADER => $this->header,
                CURLOPT_POSTFIELDS => json_encode($data),
                CURLOPT_CUSTOMREQUEST => 'DELETE',
            ));
            $response = curl_exec($ch);
            return json_decode($response);

        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    public function statuschangeCampaign($data)
    {
        // echo "<pre>";print_r($data);exit;
        $merchantId = $this->_scopeConfig->getValue(self::XML_USER_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);

        try {
            $url = $this->apiDomain . '/campaigns/update';
            $data = [
                'merchant_id' => $merchantId,
                'customer_id' => $customerId,
                'campaign_id' => $data['campaign_id'],
                'account_budget_id' => $data['account_budget_id'],
                'campaign_name' => $data['campaign_name'],
                'target_country' => $data['target_country'],
                'budget' => $data['budget'],
                'status' => $data['status'], // ENABLE => 2, PAUSED => 3, REMOVED => 4
                'all_products' => 1,
                'ad_group_id' => $data['ad_group_id'],
                'ad_group_resource_name' => $data['ad_group_resource_name'],
                'filter_by' => 'category',
                'filter_data' => json_encode(['id' => '', 'level' => ''])
            ];
            
            $this->header = array(
                "Authorization: Bearer MTIzNA==",
                "Content-Type: application/json"
            );

            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $url, //esc_url($this->curl_url),
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 10000,
                CURLOPT_HTTPHEADER => $this->header,
                CURLOPT_POSTFIELDS => json_encode($data),
                CURLOPT_CUSTOMREQUEST => 'PATCH',
            ));
            $response = curl_exec($ch);
            // print_r($data);
            // exit;
            return json_decode($response);

        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    
    /**Perfomance Max Campaing Start*/
    public function createShippingCampaignPmax($data1)
    {

        $merchantId = $this->_scopeConfig->getValue(self::XML_USER_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);

        $site_key = isset($data1['site_key'])?$data1['site_key']:"";
        $site_url = isset($data1['site_url'])?$data1['site_url']:"";
        $urls = array();
        if(!empty($site_url)){
            foreach($site_url as $key => $val){
                if(isset($site_key[$key]) && $site_key[$key] && $val){
                    $urls[]=array('key'=>$site_key[$key], 'value' => $site_url[$key]);
                }
            }
        }

        try {
            $url = $this->apiDomain.'/pmax/create';
            $data = [
                "customer_id" => $customerId,
                "merchant_id" => $merchantId,
                "campaign_name" => $data1['campaign_namePmax'],
                "budget" => $data1['campaign_budgetPmax'],
                "target_country" => $data1['target_countryPmax'],
                "status"=> $data1['status'],
                "final_url_suffix" => $data1['finalurl_suffix'],
                "urls" => $urls,
                "target_roas"=> $data1['target_ROAS'],
                "start_date" => $data1['start_datePmax'],
                "end_date" => $data1['end_datePmax'],
                "enable_local"=> true,
                "access_token" => $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken())
            ];

            // post method
            $this->curl->post($url, $data);
            $headers = ["Content-Type" => "application/json", "Authorization" => "Bearer {$this->token}"];
            $this->curl->setHeaders($headers);
            $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);
            
            // output of curl requestt
            return $this->curl->getBody();

        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    public function updateShoppingCampaignPmax($datas)
    {
        $merchantId = $this->_scopeConfig->getValue(self::XML_USER_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);

        $site_keyEdit = isset($datas['site_keyEdit'])?$datas['site_keyEdit']:"";
        $site_urlEdit = isset($datas['site_urlEdit'])?$datas['site_urlEdit']:"";
        $urlsEdit = array();
        if(!empty($site_urlEdit)){
            foreach($site_urlEdit as $key => $val){
                if(isset($site_keyEdit[$key]) && $site_keyEdit[$key] && $val){
                    $urlsEdit[]=array('key'=>$site_keyEdit[$key], 'value' => $site_urlEdit[$key]);
                }
            }
        }

        try {

            $url = $this->apiDomain.'/pmax/update';
            $data = [
                    "customer_id" => $customerId,
                    "campaign_id" => $datas['campaign_id'],
                    "resource_name"=> $datas['resource_name'],
                    "campaign_name"=> $datas['campaign_name'],
                    "budget"=> $datas['budget'],
                    "campaign_budget_resource_name"=>$datas['campaign_budget_resource_name'],
                    "status"=> $datas['status'],
                    "final_url_suffix" => $datas['final_url_suffix'],
                    "merchant_id" => $merchantId,
                    "target_country"=> $datas['target_country'],
                    "urls" => $urlsEdit,
                    //"enable_local" => true,
                    "target_roas" => $datas['target_roas'],
                    "start_date" => $datas['startDatePmax_Edit'],
                    "end_date" => $datas['endDatePmax_Edit'],
                    "access_token" => $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken())
            ];

            $this->header = array(
                "Authorization: Bearer MTIzNA==",
                "Content-Type: application/json"
            );

            $this->curl->post($url, $data);
            $headers = ["Content-Type" => "application/json", "Authorization" => "Bearer {$this->token}"];
            $this->curl->setHeaders($headers);
            $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);
            
            // output of curl requestt
            return $this->curl->getBody();

        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    public function deleteShoppingCampaignPmax($datas)
    {
        $merchantId = $this->_scopeConfig->getValue(self::XML_USER_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);

        try {
            $url = $this->apiDomain.'/pmax/delete';
            $data = [
                'customer_id' => $customerId,
                'resource_name' => $datas['resourceName'],
                'access_token' => $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken())
            ];
            
            $this->header = array(
                "Authorization: Bearer MTIzNA==",
                "Content-Type: application/json"
            );

            // post method
            $this->curl->post($url, $data);
            $headers = ["Content-Type" => "application/json", "Authorization" => "Bearer {$this->token}"];
            $this->curl->setHeaders($headers);
            $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);
            
            // output of curl requestt
            return $this->curl->getBody();

        } catch (Exception $e) {

            // echo "Error Delete";
            return $e->getMessage();
        }
    }

    public function getCampaignDetailsPmax($cmpIDPmax)
    {
        $merchantId = $this->_scopeConfig->getValue(self::XML_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);

        try {
            $url = $this->apiDomain.'/pmax/detail';
            
            $data = [
                'customer_id' => $customerId,
                'campaign_id' => $cmpIDPmax,
                'access_token' => $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken())
            ];
            $postData = json_encode($data);
            
                    $this->header = array(
                        "Authorization: Bearer MTIzNA==",
                        "content-type: application/json",
                    );

            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $url, //esc_url($this->curl_url),
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 10000,
                CURLOPT_HTTPHEADER => $this->header,
                CURLOPT_POSTFIELDS => json_encode($data),
                CURLOPT_CUSTOMREQUEST => 'POST',
            ));
            $response = curl_exec($ch);
            
            return json_decode($response);

            
            // output of curl requestt
            // return $this->curl->getBody();

        } catch (Exception $e) {
            return $e->getMessage();
        }

    }

    public function accountPerformancePmax($args)
    {
        
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);
        $url = $this->apiDomain . '/reports/account-performance';
        $data = [
            'customer_id' => $customerId,
            'from_date' => $args['from_date'],
            'to_date' => $args['to_date'],
            'customRadio' => 2
        ];

        $this->header = array(
            "Authorization: Bearer MTIzNA==",
            "content-type: application/json",
        );

        $response_body = json_encode($data);
        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_URL => $url, //esc_url($this->curl_url),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_TIMEOUT => 1000,
            CURLOPT_HTTPHEADER => $this->header,
            CURLOPT_POSTFIELDS => $response_body,
        ));
        $this->response = curl_exec($ch);
        return json_decode($this->response);

    }

    public function statuschangeCampaignPmax($data)
    {
        $merchantId = $this->_scopeConfig->getValue(self::XML_USER_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);

        try {
            $url = $this->apiDomain . '/campaigns/update';
            $data = [
                'merchant_id' => $merchantId,
                'customer_id' => $customerId,
                'campaign_id' => $data['campaign_id'],
                'account_budget_id' => $data['account_budget_id'],
                'campaign_name' => $data['campaign_name'],
                'target_country' => $data['target_country'],
                'budget' => $data['budget'],
                'status' => $data['status'], // ENABLE => 2, PAUSED => 3, REMOVED => 4
                'all_products' => 1,
                'ad_group_id' => $data['ad_group_id'],
                'ad_group_resource_name' => $data['ad_group_resource_name'],
                'filter_by' => 'category',
                'filter_data' => json_encode(['id' => '', 'level' => ''])
            ];
            
            $this->header = array(
                "Authorization: Bearer MTIzNA==",
                "Content-Type: application/json"
            );

            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $url, //esc_url($this->curl_url),
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 10000,
                CURLOPT_HTTPHEADER => $this->header,
                CURLOPT_POSTFIELDS => json_encode($data),
                CURLOPT_CUSTOMREQUEST => 'PATCH',
            ));
            $response = curl_exec($ch);
            // print_r($data);
            // exit;
            return json_decode($response);

        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    public function listShippingCampaignPmax()
    {

        $merchantId = $this->_scopeConfig->getValue(self::XML_MERCHANT_ID, 'default', $this->storeScope);
        $customerId = $this->_scopeConfig->getValue(self::XML_CUSTOMER_ID, 'default', $this->storeScope);
        try {
            $url = $this->apiDomain.'/pmax/list';
            $data = [
                "customer_id" => $customerId,
                "access_token" => $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken())
            ];

            // post method
            $this->curl->post($url, $data);
            $this->header = array(
                "Authorization:Bearer MTIzNA==",
                "content-type:application/json",
                "AccessToken:" . $this->generateAccessToken($this->getAccessToken(), $this->getRefreshTOken()),
            );
            $this->curl->setHeaders($this->header);
            $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);

            // output of curl requestt
            return $this->curl->getBody();

        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    public function getCountriesPmax()
    {
        $countries = __DIR__ . '/Json/countries.json';
        $country = file_get_contents($countries);
        return $country ? json_decode($country, true) : [];
    }
    /**Perfomance Max Campaign End*/


}
